#ifndef SMBZ_ENGINE_H
#define SMBZ_ENGINE_H

#include <string>
#include <map>

#include <ulib/ulib.h>
#include <mp3_shared.h>

#include "gamecore.h"
#include "player.h"

struct StageInfo
{
	std::string name;
	std::vector<BgInfo> bgs;
	int bgcolor;
	struct
	{
		UL_IMAGE* image;
		bool on;
		int offsetY;
		Rect topFrame;
		Rect loopFrame;
	} ground;
};
typedef struct StageInfo StageInfo;

class Engine
{
public:
	Engine();
	void setFighter(int ind, std::string name, bool isLocal);
	void setStage(std::string name);

	void playSnd(std::string name);
	void setTimeScale(float scale, int autoreset, Callback newAction);
	void moveToFront(int ind);

	void start();
	void stop();
	void tick();
	void draw();


	bool isStarted() {return started;}
	bool fightStarted() {return countDownTicks[0] <= 0;}
	StageInfo* getStage() {return &stage;}
	Player* getPlayer(int i) {return players[i];}
	float CamX() {return camX;}
	float CamY() {return camY;}
	int gameOverTicks() {return gameOverTick;}

	int fadeAdd, darkbgAdd;
	std::map<std::string, volatile wav_stuffs*> snds;

private:
	bool started;

	float ticks;
	int fadeAlpha, darkbgAlpha;
	int countDownTicks[2];
	int gameOverTick;

	float timeScale;
	int timeAutoReset;
	Callback timeCallback;

	float camX, camY, camZoom;
	std::vector<Player*> camFollow;

	Player* players[2];
	Player* playerDrawOrder[2];
	StageInfo stage;
};

#endif