#ifndef SMBZ_PLAYER_H
#define SMBZ_PLAYER_H

#include <string>

#include "fighterinfo.h"
#include "engine.h"

struct PlayerInputs
{
	int dirX, dirY;
	bool jump;
	bool guard;
	bool attack;
	bool attackHold;
	bool trigger;
	int attackTicks;
	bool attackType;
};

struct AttackBox
{
	bool on, deleteOnNextFrame;
	int x, y, w, h;
	ActionCallback onHit;
};

class Player
{
public:
	Player(int _ind, Engine* engine) {ind = _ind; m_engine = engine;}
	void setFighter(std::string name, bool _isLocal);

	void init(int hp=150, bool reuse=false);
	void destroy();
	void tick(float dt);
	void draw(float screenW, float screenH);

	void facePlayer(Player* who);
	void giveDamage(float hp, int addStun, bool guard=false);
	void knockBack(float x, float y, Player* other, bool guard=false, bool absoluteVel=false);
	void setAttackBox(bool on, int x, int y, int w, int h, bool deleteOnNextFrame, ActionCallback onHit);
	void setTimeScale(float scale, int autoreset);
	void playSnd(std::string name);
	void setAnim(std::string name, bool restart=false, bool dontDisableAttack=false);

	FighterInfo* getInfo() {return &info;}
	PlayerInputs* Inputs() {return &inputs;}
	AttackBox* attackBox() {return &attackbox;}
	float getHP() {return m_hp;}
	float getMaxHP() {return m_maxhp;}
	float getPosX() {return posX;}
	float getPosY() {return posY;}
	float getVelX() {return velX;}
	float getVelY() {return velY;}
	int getDir() {return dir;}
	void setPos(float x, float y) {posX = x; posY = y;}
	void setVel(float x, float y) {velX = x; velY = y;}
	void setRotation(float New) {rot = New;}
	UL_IMAGE* getSheet(std::string name) {return sheets[name];}
	bool isAttacking() {return attacking;}
	bool isGuarding() {return guarding;}
	int Ind() {return ind;}
	int getStun() {return stun;}
	int getGuardStun() {return guardStun;}
	void setAttacking(bool attack) {attacking = attack;}
	int HitsFrom() {return hitsFrom;}

	struct
	{
		ActionCallback callback;
		Player* enemy;
		u32 guardTick;
		u32 tactic;
		u32 tacticState;
		u32 tacticSwitch;
		u32 attackTick;
		u32 releaseAttack;
	} cpu;

private:
	Engine* m_engine;
	FighterInfo info;
	bool isLocal, enabled, m_reuse;
	float m_hp, m_maxhp;
	float posX, posY;
	float velX, velY;
	int ind;

	bool guarding;
	bool attacking;
	std::string attackName;
	int dir;
	float stun;
	float guardStun;
	int hitsTo;
	int hitsFrom;
	float rot;

	std::map<std::string, UL_IMAGE*> sheets;
	std::map<std::string, volatile wav_stuffs*> sounds;

	float timeScale;
	int timeAutoReset;

	AttackBox attackbox;
	PlayerInputs inputs;
};

#endif