/*
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define ds_color_c
#define LUA_LIB

#include "lua.h"
#include "lauxlib.h"
#include "lualib.h"

#include <ulib/ulib.h>

#include "vars.h"

float hue2rgb(float p, float q, float t)
{
	if (t < 0) t++;
	if (t > 1) t--;
	if (t < 1./6) return p + (q - p) * 6 * t;
	if (t < 1./2) return q;
	if (t < 2./3) return p + (q - p) * (2./3 - t) * 6;
	return p;
}

static int color_new(lua_State *L){
    int r = (int)luaL_checknumber(L, 1);
    int g = (int)luaL_checknumber(L, 2);
    int b = (int)luaL_checknumber(L, 3);
    assert(L, r>=0 && r<=31, "Red mask must be between 0 and 31")
    assert(L, g>=0 && g<=31, "Green mask must be between 0 and 31")
    assert(L, b>=0 && b<=31, "Blue mask must be between 0 and 31")
    lua_pushnumber(L, RGB15(r, g, b));
    return 1;
}

static int color_new256(lua_State *L) {
    int r = (int)luaL_checknumber(L, 1);
    int g = (int)luaL_checknumber(L, 2);
    int b = (int)luaL_checknumber(L, 3);
    assert(L, r>=0 && r<=255, "Red mask must be between 0 and 255");
    assert(L, g>=0 && g<=255, "Green mask must be between 0 and 255");
    assert(L, b>=0 && b<=255, "Blue mask must be between 0 and 255");
    r = floor(r * 31 / 255);
    g = floor(g * 31 / 255);
    b = floor(b * 31 / 255);
    lua_pushnumber(L, RGB15(r, g, b));
    return 1;
}

static int color_newHSL(lua_State *L)
{
	float h = (float)luaL_checknumber(L, 1);
    float s = (float)luaL_checknumber(L, 2);
    float l = (float)luaL_checknumber(L, 3);
    assert(L, h>=0 && h<=1, "Hue must be between 0 and 1.");
    assert(L, s>=0 && s<=1, "Saturation must be between 0 and 1.");
    assert(L, l>=0 && l<=1, "Lightness must be between 0 and 1.");

	int r, g, b;

	if (s == 0)
	{
		// achromatic
		r = l * 31;
		g = l * 31;
		b = l * 31;
	}
	else
	{
		float q = (l < 0.5) ? (l * (1 + s)) : (l + s - l * s);
		float p = 2 * l - q;
		
		r = hue2rgb(p, q, h + 1./3) * 31;
		g = hue2rgb(p, q, h) * 31;
		b = hue2rgb(p, q, h - 1./3) * 31;
	}

	lua_pushnumber(L, RGB15(r, g, b));

	return 1;
}

static const luaL_Reg colorlib[] = {
    {"new", color_new},
    {"new256", color_new256},
	{"newHSL", color_newHSL},
    {NULL, NULL}
};

/*
** Open infos library
*/
LUALIB_API int luaopen_color(lua_State *L) {
    luaL_register(L, LUA_COLORLIBNAME, colorlib);
    return 1;
}
