#include <stdlib.h>
#include <stdio.h>

#define ds_mp3_c
#define LUA_LIB

#include "lua.h"
#include "lauxlib.h"
#include "lualib.h"

#include <ulib/ulib.h>
#include "mp3_shared.h"
#include <sys/stat.h>
#include <sys/dir.h>

#include "vars.h"

static int luamp3_init(lua_State *L)
{
	return mp3_init();
}

static int luamp3_play(lua_State *L)
{
	char* filename = (char*)luaL_checkstring(L, 1);
	bool loop = (lua_isboolean(L, 2)) ? (bool)lua_toboolean(L, 2) : false;
	float loopsec = luaL_optnumber(L, 3, 0.f);
    assert(L, filename != NULL, "You must enter an MP3 filename.");
	assert(L, loopsec >= 0, "Loop second must be 0 or higher.");

    struct stat file_status;
    if(stat(filename, &file_status) != 0)
	{
        luaL_error(L, "File '%s' doesn't exist.", filename);
		return 0;
	}

	int err = mp3_play(filename, loop, loopsec);
	if (err < 0)
		luaL_error(L, "MP3 error %d", err);

	return 0;
}

static int luamp3_pause(lua_State *L)
{
	mp3_pause();
	return 0;
}

static int luamp3_resume(lua_State *L)
{
	mp3_resume();
	return 0;
}

static int luamp3_stop(lua_State *L)
{
	mp3_stop();
	return 0;
}

static int luamp3_set_volume(lua_State *L)
{
	int vol = luaL_checknumber(L, 1);
	assert(L, vol >= 0 && vol <= 255, "Volume must be a number between 0-255.");

	mp3_set_volume(vol);
	return 0;
}

static int luamp3_fill_buffer(lua_State *L)
{
	mp3_fill_buffer();
	return 0;
}

static int luamp3_is_playing(lua_State *L)
{
	lua_pushboolean(L, mp3_is_playing());
	return 1;
}

static const luaL_Reg mp3lib[] = {
	{"init", luamp3_init},
	{"play", luamp3_play},
	{"pause", luamp3_pause},
	{"resume", luamp3_resume},
	{"stop", luamp3_stop},
	{"setVolume", luamp3_set_volume},
	{"isPlaying", luamp3_is_playing},
	{"fillBuffer", luamp3_fill_buffer},
    {NULL, NULL}
};


LUALIB_API int luaopen_mp3 (lua_State *L) {
    luaL_register(L, LUA_MP3LIBNAME, mp3lib);
    return 1;
}
