/*
*
*/

#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/dir.h>

#include <ulib/ulib.h>
#include <maxmod9.h>

#include <microcpp/sound.h>
#include <microcpp/vars.h>

namespace Sound
{
	// ########## BANKS ##########

	void loadBank(const char* filename)
	{
		struct stat file_status;
		if(stat(filename, &file_status) != 0) return 0;
		mmInitDefault(filename);
	}

	void unloadBank()
	{
		mmInitDefaultMem((mm_addr)0);
	}

	// ########## MODS ##########

	void loadMod(int mod)
	{
		mmLoad(mod);
	}

	void unloadMod(int mod)
	{
		mmUnload(mod);
	}

	void startMod(int mod, int mode)
	{
		mmStart(mod, (mode == 0) ? MM_PLAY_LOOP : MM_PLAY_ONCE);
	}

	void pause()
	{
		mmPause();
	}

	void resume()
	{
		mmResume();
	}

	void stop()
	{
		mmStop();
	}

	void setPosition(int pos)
	{
		mmPosition(pos);
	}

	bool isActive()
	{
		return mmActive();
	}

	void startJingle(int mod)
	{
		mmJingle(mod);
	}

	void setModVolume(int vol)
	{
		mmSetModuleVolume(vol);
	}

	void setJingleVolume(int vol)
	{
		mmSetJingleVolume(vol);
	}

	void setModTempo(int tempo)
	{
		mmSetModuleTempo(tempo);
	}

	void setModPitch(int pitch)
	{
		mmSetModulePitch(pitch);
	}

	// ########## SFX ##########

	void loadSFX(int sfx)
	{
		mmLoadEffect(sfx);
	}

	void unloadSFX(int sfx)
	{
		mmUnloadEffect(sfx);
	}

	mm_sfxhand startSFX(int sfx) // mm_sfxhand is also u16
	{
		return mmEffect(sfx);
	}

	static int sound_setSFXVolume(lua_State *L)
	{
		int handle = luaL_checknumber(L, 1);
		int vol = luaL_checknumber(L, 2);
		assert(L, vol >= 0 && vol <= 255, "Volume must be between 0 and 255");
		mmEffectVolume(handle, vol);
		return 0;
	}

	static int sound_setSFXPanning(lua_State *L)
	{
		int handle = luaL_checknumber(L, 1);
		int pan = luaL_checknumber(L, 2);
		assert(L, pan >= 0 && pan <= 255, "Panning must be between 0 and 255");
		mmEffectPanning(handle, pan);
		return 0;
	}

	static int sound_setSFXPitch(lua_State *L)
	{
		int handle = luaL_checknumber(L, 1);
		int pitch = luaL_checknumber(L, 2);
		mmEffectPanning(handle, pitch);
		return 0;
	}

	static int sound_setSFXScalePitch(lua_State *L)
	{
		int handle = luaL_checknumber(L, 1);
		int factor = luaL_checknumber(L, 2);
		mmEffectScaleRate(handle, factor);
		return 0;
	}

	static int sound_stopSFX(lua_State *L)
	{
		int handle = luaL_checknumber(L, 1);
		mmEffectCancel(handle);
		return 0;
	}

	static int sound_releaseSFX(lua_State *L)
	{
		int handle = luaL_checknumber(L, 1);
		mmEffectRelease(handle);
		return 0;
	}

	static int sound_stopAllSFX(lua_State *L)
	{
		mmEffectCancelAll();
		return 0;
	}
}
