#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>

#include <nds/ipc.h>
#include <nds.h>
#include <fat.h>

#include <ulib/ulib.h>

#include <microcpp/system.h>
#include <microcpp/vars.h>

#define lua_geti(L, index, i) \
    lua_pushnumber(L, i); \
    lua_gettable(L, index);
#define lua_seti(L, index, i) \
    lua_pushnumber(L, i); \
    lua_pushvalue(L, -2); \
    lua_settable(L, index); \
    lua_pop(L, 1);
#define isRelative(path) \
    path[0] != '/' && \
    !strncmp("nitro:/", path, 5) && !strncmp("fat:/", path, 5)

namespace System
{
	int currentVramUsed()
	{
		return ulGetTexVramUsedMemory();
	}

	int currentVramFree()
	{
		return ulGetTexVramAvailMemory();
	}

	// This function returns a bizarre number
	int currentPalUsed()
	{
		return ulGetTexPalUsedMemory();
	}

	int currentPalFree()
	{
		return ulGetTexPalAvailMemory();
	}

	u32 currentVertexFree()
	{
		lua_pushnumber(L, (unsigned int) ulGetVertexAvailMemory());
		return 1;
	}

	std::string currentDirectory(lua_State *L)
	{
		char path[256];
		getcwd(path, 255);
		return path;
	}

	void changeDirectory(const char* dir)
	{
		chdir(dir);
	}

	void remove(const char* dir)
	{
		remove(dir);
	}

	void rename(const char* old, const char* new){
		rename(old, new);
	}

	void makeDirectory(const char* dir)
	{
		mkdir(dir, 777);
	}

	bool fileExists(const char* file)
	{
		struct stat st;
		return stat(file, &st) == 0;
	}

	/*
	int mustSwap(lua_State *L, int dirList, int i) {
		// Special function to sort the final table placing folders first
		int isDir1, isDir2, nameCmp;
		char *name1 = malloc(256*sizeof(char)), *name2 = malloc(256*sizeof(char));
		
		lua_geti(L, dirList, i);
		lua_getfield(L, -1, "isDir");
		isDir1 = lua_toboolean(L, -1);
		lua_getfield(L, -2, "name");
		strcpy(name1, lua_tostring(L, -1));
		lua_pop(L, 3);                      // Removes elem[i], its 'isDir' and its 'name'
		lua_geti(L, dirList, i+1);
		lua_getfield(L, -1, "isDir");
		isDir2 = lua_toboolean(L, -1);
		lua_getfield(L, -2, "name");
		strcpy(name2, lua_tostring(L, -1));
		lua_pop(L, 3);                      // Idem
		
		nameCmp = strcmp(strlwr(name1), strlwr(name2));
		
		// hey guys, you forgot these lines so i added them for you
		free(name1);
		free(name2);
		name1 = NULL;
		name2 = NULL;
		
		return (!isDir1 && isDir2) || ((nameCmp > 0) && (isDir1 == isDir2));
	}
	*/

	std::vector<std::string> system_listDirectory(const char* askedPath)
	{
		char path[PATH_MAX] = "";
		struct dirent *entry;
		struct stat st;
		DIR *dir;
		// entryName points to the end of the absolute path leading to the folder (and to the start of the name of the entry in the path)
		char *entryName, *end = path + sizeof(path) - 1;
		int i = 1, nbItems;
		int dirList;            // Hold the index of the resulting table in the stack
		int swapped = 1;

		std::vector<std::string> dirs, files;
		
		if (isRelative(askedPath) && getcwd(path, sizeof(path)) == NULL) return files; // "couldn't get the current working directory"
		if (strlen(path) + strlen(askedPath) >= sizeof(path)) return files; // "path is too long"
		strcpy(path + strlen(path), askedPath);
		
		// We will append each entry's name there to the path
		entryName = path + strlen(path);
		
		// Make sure there's a trailing slash
		if (entryName[-1] != '/' && entryName < end)
			*entryName++ = '/';     // Also move the beginning of the entry's name to the right
		
		if ((dir = opendir(askedPath)) == NULL) // "couldn't open folder %s", askedPath
		
		lua_newtable(L);        // This is the return table listing the folder
		dirList = lua_gettop(L);
		
		while ((entry = readdir(dir)) != NULL) {    // Loop through the folder
			if (entryName + strlen(entry->d_name) >= end) luaL_error(L, "path is too long");
			
			strcpy(entryName, entry->d_name);
			
			if (stat(path, &st) != 0) luaL_error(L, "couldn't stat the entry %s", entry->d_name);
			
			// From now everything is okay
			lua_pushnumber(L, i);                   // Will be the index of this element
			lua_newtable(L);                        // Entry's table
			// Fill the element
			lua_pushstring(L, entry->d_name);
			lua_setfield(L, -2, "name");
			lua_pushboolean(L, entry->d_type == DT_DIR);
			lua_setfield(L, -2, "isDir");
			lua_pushnumber(L, st.st_size);
			lua_setfield(L, -2, "size");
			// Add the element to the resulting table
			lua_settable(L, dirList);
			i++;
		}
		
		closedir(dir);
		
		nbItems = i - 1;

		/*
		// Now we need to sort the table (we will use modified bubble sorting)
		// We use a custom function to put folders at the top of the table
		while (swapped) {
			swapped = 0;
			
			for (i = 1; i <= nbItems-1; i++) {
				if (mustSwap(L, dirList, i)) {
					lua_geti(L, dirList, i);
					lua_geti(L, dirList, i+1);
					lua_seti(L, dirList, i);
					lua_seti(L, dirList, i+1);
					swapped = 1;
				}
			}
		}
		*/
		
		return 1;
	}

	void system_setLedBlinkMode(int mode)
	{
		ledBlink(mode);
	}

	void systemShutDown()
	{
		systemShutDown();
	}

	void systemSleep()
	{
		systemSleep();
	}
}