#include <stdlib.h>
#include <nds/ndstypes.h>

#include <microcpp/microcpp.h>
#include <fighterinfo.h>
#include <player.h>

void pause4(const char* text)
{
	for (int i=0; i<60; i++)
	{
		screen::print(0, 8, 8, text);
		screen::render();
	}
}

std::map<std::string, ActionCallback> actionfuncs;



// action callbacks START

// generic callbacks used in all characters
void genericGuard(Engine* engine, Player* player, Player* other, bool guard)
{
	if (!player->Inputs()->guard && player->getPosY() < 0) player->setAnim("fall");
}

void genericToFalling(Engine* engine, Player* player, Player* other, bool guard)
{
	if (player->getVelY() > 0) player->setAnim("fall");
}

void genericToFallNow(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAnim("fall");
}

void genericHurt(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAnim((player->getPosY() < 0) ? "airhurt" : "hurtground");
}

void genericReeling(Engine* engine, Player* player, Player* other, bool guard)
{
	if (player->getPosY() >= 0) player->setAnim("painful_landing");
	else if (player->getStun() <= 0) player->setAnim("fall");
}

void genericToReeling(Engine* engine, Player* player, Player* other, bool guard)
{
	if (player->getVelY() >= 0) player->setAnim("reeling");
}

void genericMissB(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missB");
}

void genericMissLight(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missLight");
}

void genericSwing(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("swing");
}

void genericAirRecover(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("airrecover");
}

void genericStopSpin(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("stopspin");
}

void genericResetRotation(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setRotation(0);
}

void genericStopAttacking(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttacking(false);
}


// Mario hit callbacks
void marioJabComboHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitWeakA");
	other->knockBack(50, 0, player, guard, true);
	other->giveDamage(1, 15, guard);
}

void marioUppercutHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(50, -500, player, guard, true);
	other->giveDamage(1, 30, guard);
}

void marioHookPunchHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(-100, -175, player, guard);
	other->giveDamage(3, 20, guard);
}

void marioSmackDownHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(100, 250, player, guard);
	other->giveDamage(1.5, 45, guard);
}

void marioAirUppercutHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(50, -500, player, guard);
	other->giveDamage(2, 30, guard);
}

void marioEaglekickHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd("hitWeakB");
	other->knockBack(125, 0, player, guard);
	other->giveDamage(1, 25, guard);
	player->setAnim("eaglekick2");
	player->setVel(100 * -player->getDir() + player->getVelX(), -400);
}

void marioGroundpoundHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(0, 600, player, guard);
	other->giveDamage(2, 30, guard);
	if (other->getVelY() < 0) other->setVel(other->getVelX(), 0);
}

void marioGroundpoundLandHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitStrongA");
	other->knockBack(125, -400, player, guard);
	other->giveDamage(3, 25, guard);
}

void marioSjpHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(50, -400, player, guard, true);
	other->giveDamage(0.84, 15, guard);
}

void marioHammerhopHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(100, -300, player, guard, true);
	other->giveDamage(3, 33, guard);
}

void marioHammerSmashHit(Engine* engine, Player* player, Player* other, bool guard)
{
	// ignore guard
	engine->playSnd("hitStrongA");
	other->knockBack(175, -300, player);
	other->giveDamage(6, 60);
}

void marioHammerMeteorHit(Engine* engine, Player* player, Player* other, bool guard)
{
	// ignore guard
	engine->playSnd("hitStrongB");
	other->knockBack(200, 350, player);
	other->giveDamage(4, 60);
}


// Mario callbacks
void marioJump(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setVel(player->getVelX(), -500);
	player->playSnd("jump");
}

void marioPainfulLanding(Engine* engine, Player* player, Player* other, bool guard)
{
	if (player->getPosY() < 0) player->setAnim("reeling");
	else if (player->getStun() <= 15) player->setAnim("recover");
}

void marioJabComboHitFinal(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(210, -75, player, guard, true);
	other->giveDamage(2, 15, guard);
}

void marioJabCombo1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missLight");
	player->setAttackBox(true, 16, 8, 16, 26, true, marioJabComboHit);
}

void marioJabCombo2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missLight");
	player->setAttackBox(true, 21, 0, 19, 24, true, marioJabComboHit);
}

void marioJabCombo3(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missA");
	player->setAttackBox(true, 17, 12, 22, 14, true, marioJabComboHitFinal);
}

void marioUppercut(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missLight");
	player->setAttackBox(true, 12, 0, 11, 18, true, marioUppercutHit);
}

void marioHookPunch(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 22, 0, 18, 24, true, marioHookPunchHit);
}

void marioSmackDown(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missB");
	player->setAttackBox(true, 18, -1, 19, 38, true, marioSmackDownHit);
}

void marioAirUppercut(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missLight");
	player->setAttackBox(true, 16, 0, 16, 16, true, marioAirUppercutHit);
}

void marioEaglekick(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missC");
	player->setAttackBox(true, 10, 24, 15, 15, true, marioEaglekickHit);
}

void marioPreGroundpound(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("spinjump");
	player->setVel(player->getVelX(), -350);
}

void marioGroundpound(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 5, 24, 20, 10, false, marioGroundpoundHit);
	player->setAnim("groundpound2", false, true);
}

void marioGroundpoundToLand(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setVel(0, 650);
	if (player->getPosY() >= 0) player->setAnim("groundpound_land");
}

void marioGroundpoundLand(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 5, 22, 22, 9, true, marioGroundpoundLandHit);
}

void marioSjp1(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd("missB");
	if (rand() % 100 >= 70) player->playSnd("strain");
	player->setAttackBox(true, 5, 13, 24, 20, true, marioSjpHit);
}

void marioSjp2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setVel(50 * player->getDir(), -475);
	player->setAttackBox(true, 15, 0, 23, 37, true, marioSjpHit);
}

void marioSjp3(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 10, -4, 21, 21, true, marioSjpHit);
}

void marioSjp4(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 10, -4, 16, 17, true, marioSjpHit);
}

void marioHammerhop0(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setVel(325 * player->getDir(), -125);
}

void marioHammerhop1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 9, 0, 32, 33, false, marioHammerhopHit);
}

void marioHammerhop2(Engine* engine, Player* player, Player* other, bool guard)
{
	// don't turn attackbox on again if player was already hit (1st argument)
	player->setAttackBox(player->attackBox()->on, 15, 0, 22, 31, true, marioHammerhopHit);
}

void marioSndStrain(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("strain");
}

void marioHammerSmash1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, -1, -1, 21, 28, false, marioHammerSmashHit);
}

void marioHammerSmash2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 0, 0, 41, 24, false, marioHammerSmashHit);
}

void marioHammerSmash3(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 0, 0, 40, 48, false, marioHammerSmashHit);
}

void marioHammerSmash4(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 18, 0, 24, 46, false, marioHammerSmashHit);
}

void marioHammerSmash5(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 21, 0, 21, 36, false, marioHammerSmashHit);
}

void marioPreHammerMeteor(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setRotation(0);
	player->playSnd("eeeiiii");
}

void marioHammerMeteor(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd("missB");
	player->playSnd("ha");
	player->setAttackBox(true, 31, 0, 52, 78, true, marioHammerMeteorHit);
}

void marioPostHammerMeteor(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAnim("hammermeteor2");
}

void marioHammerMeteorLand(Engine* engine, Player* player, Player* other, bool guard)
{
	if (player->getPosY() >= 0) player->setAnim("hammermeteor_land");
}



// Sonic hit callbacks
void sonicJabComboHit1(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitWeakA");
	other->knockBack(50, 0, player, guard, true);
	other->giveDamage(1, 15, guard);
}

void sonicJabComboHit2(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(210, -300, player, guard);
	other->giveDamage(2, 20, guard);
}

void sonicSweepKick1Hit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(210, -300, player, guard);
	other->giveDamage(3, 50, guard);
}

void sonicSweepKick2Hit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(-210, -300, player, guard);
	other->giveDamage(3, 50, guard);
}

void sonicFlipKickHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(50, -500, player, guard);
	other->giveDamage(2, 30, guard);
}

void sonicTornadoKickHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitWeakA");
	other->knockBack(50, -225, player, guard);
	other->giveDamage(1.8, 30, guard);
	if (other->HitsFrom() < 4)
	{
		player->setVel(other->getVelX(), other->getVelY());
	}
}

void sonicDownKickHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(150, 400, player, guard);
	other->giveDamage(2, 30, guard);
}

void sonicSpinDashSlideHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(100, -500, player, guard, true);
	other->giveDamage(1.5, 30, guard);
}

void sonicSpinDashBoostHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(100, -500, player, guard, true);
	other->giveDamage(2.5, 30, guard);
}

void sonicFlashKickHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(50, -425, player, guard);
	other->giveDamage(2, 45, guard);
}

void sonicEagleHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(200, 400, player, guard);
	other->giveDamage(2, 40, guard);
}

void sonicDiveKickHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	if (other->getPosY() >= 0) other->knockBack(75, -400, player, guard);
	else other->knockBack(75, 650, player, guard);
	other->giveDamage(4, 45, guard);
}



// Sonic callbacks
void sonicJump(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setVel(player->getVelX(), -500);
	player->playSnd("jump");
	player->setAnim("jump2");
}

void sonicPainfulLanding(Engine* engine, Player* player, Player* other, bool guard)
{
	if (player->getPosY() < 0) player->setAnim("reeling");
	else if (player->getStun() <= 30) player->setAnim("recover");
}

void sonicJabCombo1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missLight");
	player->setAttackBox(true, 21, 2, 18, 25, true, sonicJabComboHit1);
}

void sonicJabCombo2a(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missLight");
	player->setAttackBox(true, 13, 13, 14, 20, false, sonicJabComboHit1);
}

void sonicJabCombo2b(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 13, 10, 27, 23, true, sonicJabComboHit1);
}

void sonicJabCombo3a(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missA");
	player->setAttackBox(true, 16, 13, 19, 22, false, sonicJabComboHit2);
}

void sonicJabCombo3b(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 19, 1, 15, 26, true, sonicJabComboHit2);
}

void sonicSweepKick1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 21, 4, 20, 24, true, sonicSweepKick1Hit);
}

void sonicSweepKick2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, -1, 6, 18, 22, true, sonicSweepKick2Hit);
}

void sonicSweepKickJump(Engine* engine, Player* player, Player* other, bool guard)
{
	if (player->Inputs()->jump)
	{
		player->setVel(player->getVelX(), -500);
		player->setAnim("sweepkick_jump");
		player->setAttacking(false);
	}
}

void sonicFlipKick1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 20, 0, 20, 39, false, sonicFlipKickHit);
}

void sonicFlipKick2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 24, 0, 21, 38, true, sonicFlipKickHit);
}

void sonicTornadoKick1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 28, 6, 21, 23, true, sonicTornadoKickHit);
}

void sonicTornadoKick2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAnim("fall");
	player->setAttacking(false);
}

void sonicDownKick1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("missLight");
	player->setAttackBox(true, 14, 17, 23, 25, false, sonicDownKickHit);
}

void sonicDownKick2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 18, 18, 18, 21, true, sonicDownKickHit);
}

void sonicSpinSound(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("spin");
}

int chargePower = 0;
void sonicStopDashing(Engine* engine, Player* player, Player* other, bool guard)
{
	if (!player->Inputs()->attackHold || player->getInfo()->frame >= player->getInfo()->anims[player->getInfo()->anim].frames.size()-1)
	{
		chargePower = (player->getInfo()->frame >= 17) ? 1 : 0;
		player->setAnim((player->getInfo()->frame >= 12) ? "spindash_boost" : "spindash_slide1");
	}
}

void sonicSpinDashSlide1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("ya");
	player->setVel(450*player->getDir(), player->getVelY());
	player->setAnim("spindash_slide2");
}

void sonicSpinDashSlide2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 22, 0, 19, 26, false, sonicSpinDashSlideHit);
}

void sonicSpinDashBoost1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("zoom");
	player->setVel((2500 + (1500*chargePower)) * player->getDir(), player->getVelY());
	player->setAttackBox(true, 59, 0, 58, 34, true, sonicSpinDashBoostHit);
}

void sonicSpinDashBoost2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setVel(300 * player->getDir(), player->getVelY());
}

void sonicFlashKick1(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd("swing");
	player->playSnd("humm");
	player->setVel(player->getVelX(), -500);
}

void sonicFlashKick2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 16, 8, 16, 26, false, sonicFlashKickHit);
}

void sonicFlashKick3(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(player->attackBox()->on, 16, 8, 16, 26, true, sonicFlashKickHit);
}

void sonicBounceHit(Engine* engine, Player* player, Player* other, bool guard)
{
	engine->playSnd(guard ? "hitWeakB" : "hitMediumA");
	other->knockBack(100, -575, player, guard);
	other->giveDamage(3, 40, guard);
}

void sonicBounce(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttacking(false);
	player->playSnd("spring");
	player->setVel(player->getVelX(), -600);
	player->setAttackBox(true, -1, 0, 21, 43, true, sonicBounceHit);
}

void sonicToJump2(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAnim("jump2");
}

void sonicBounceAir(Engine* engine, Player* player, Player* other, bool guard)
{
	if (player->getPosY() >= 0)
	{
		player->setAnim("bounce");
		player->setAttacking(true);
	}
}

void sonicEagle1(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setAttackBox(true, 19, 0, 17, 38, true, sonicEagleHit);
}

void sonicPreDiveKick(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setVel(player->getVelX(), -350);
}

void sonicDiveKickStart(Engine* engine, Player* player, Player* other, bool guard)
{
	player->playSnd("chea");
	player->setAnim("divekick2");
	player->setAttackBox(true, 14, 17, 23, 25, false, sonicDiveKickHit);
}

void sonicDiveKickMove(Engine* engine, Player* player, Player* other, bool guard)
{
	player->setVel(500*player->getDir(), 650);
	if (player->getPosY() >= 0)
	{
		player->setAttacking(false);
		player->attackBox()->on = false;
	}
}

void marioCPU(Engine* engine, Player* player, Player* other, bool guard) // other is CPU enemy player
{
	float xdiff = other->getPosX() - player->getPosX();
	float xdist = abs(xdiff);
	int xdir = (xdiff >= 0) ? 1 : -1;
	float ydiff = other->getPosY() - player->getPosY();
	float ydist = abs(ydiff);
	int ydir = (ydiff >= 0) ? 1 : -1;

	bool avoidPlayer = (other->isAttacking() && other->getInfo()->anims[other->getInfo()->anim].breakthrough && xdist < 64);
	if (avoidPlayer && player->cpu.guardTick > 10)
		player->cpu.guardTick = rand() % 10;

	if (player->getStun() > 10)
	{
		player->cpu.guardTick = 15 + (rand() % 60);
		player->Inputs()->attack = false;
	}
	else if (player->getGuardStun() > 0)
	{
		player->cpu.guardTick = 10 + (rand() % 20);
		player->Inputs()->attack = false;
	}
	else if (player->cpu.guardTick > 0)
	{
		player->cpu.guardTick--;
		player->Inputs()->attack = false;
	}
	else
	{
		player->Inputs()->dirX = (avoidPlayer) ? -xdir : (xdist > 16 || player->getDir() != xdir) ? xdir : 0;
	}

	player->cpu.tacticSwitch--;
	if (player->cpu.tacticSwitch <= 0)
	{
		player->cpu.tactic = rand() % 7;
		player->cpu.tacticSwitch = 60 + (rand() % 30);
	}

	FighterAnim* anim = &player->getInfo()->anims[player->getInfo()->anim];
	player->Inputs()->jump = false;
	if (player->cpu.tactic == 0 && xdist <= 40 && ydist < 32) // jab combo
	{
		player->Inputs()->attack = true;
		player->Inputs()->dirY = 0;
		player->Inputs()->trigger = false;
		player->Inputs()->jump = rand() % 53 >= 46;
		//if (player->cpu.tacticSwitch > 0) player->cpu.tacticSwitch--;
	}
	else if (player->cpu.tactic == 1 && xdist <= 40 && ydist < 32) // jab then hook
	{
		player->Inputs()->attack = true;
		player->Inputs()->trigger = false;
		player->Inputs()->dirY = (player->getInfo()->anim == "jabcombo2" && player->getInfo()->frame >= anim->combo.frame-1) ? 1 : 0;
		u32 nextt = (player->getInfo()->anim == "hookpunch") ? rand() % 50 : 0;
		if (nextt >= 40)
		{
			player->cpu.tactic = 3; // superjumppunch
			player->cpu.tacticSwitch = 90;
		}
		else if (nextt >= 30)
		{
			player->cpu.tactic = 2; // uppercut
			player->cpu.tacticSwitch = 90;
		}
	}
	else if (player->cpu.tactic == 2 || (xdist < 32 && ydist < 32)) // uppercut
	{
		player->Inputs()->dirY = -1;
		player->Inputs()->trigger = false;
		player->Inputs()->attack = true;
		if (rand() % 50 >= 40)
		{
			player->cpu.tactic = rand() % 7;
			player->cpu.tacticSwitch = 90;
		}
	}
	else if (player->cpu.tactic == 3 && xdist <= 40 && ydist < 56) // superjumppunch
	{
		player->Inputs()->dirY = -1;
		player->Inputs()->trigger = true;
		player->Inputs()->attack = true;
	}
	else if (player->cpu.tactic == 4 && xdist < 128 && ydist < 48) // hammer hop
	{
		player->Inputs()->dirY = 0;
		player->Inputs()->trigger = true;
		player->Inputs()->attack = true;
		player->Inputs()->jump = rand() % 53 >= 46; // randomly transform into hammer meteor smash
	}
	else if (player->cpu.tactic == 5 && xdist < 48) // hammer smash
	{
		player->Inputs()->dirY = 1;
		player->Inputs()->trigger = true;
		player->Inputs()->attack = true;
		player->cpu.tactic = 0;
		player->cpu.tacticSwitch = 100;
	}
	else if (player->cpu.tactic == 6 && xdist < 104) // ground pound
	{
		player->Inputs()->jump = true;
		player->Inputs()->trigger = false;
		player->Inputs()->attack = false;
		if (ydir > 0 && xdist < 60) // we're above
		{
			player->Inputs()->dirY = 1;
			player->Inputs()->trigger = true;
			player->Inputs()->attack = true;
			if (rand() % 30 >= 20)
			{
				player->cpu.tactic = rand() % 7;
				player->cpu.tacticSwitch = 90;
			}
		}
	}
	else
	{
		player->Inputs()->attack = false;
		player->Inputs()->trigger = false;
	}

	if (player->Inputs()->attack)
	{
		if (player->cpu.attackTick <= 0) player->cpu.attackTick = 5;
		else
		{
			player->cpu.attackTick--;
			player->Inputs()->attack = false;
		}
	}
	else
		player->cpu.attackTick = 8;

	player->Inputs()->guard = (player->cpu.guardTick > 0);
}

void sonicCPU(Engine* engine, Player* player, Player* other, bool guard) // other is CPU enemy player
{
	float xdiff = other->getPosX() - player->getPosX();
	float xdist = abs(xdiff);
	int xdir = (xdiff >= 0) ? 1 : -1;
	float ydiff = other->getPosY() - player->getPosY();
	float ydist = abs(ydiff);
	int ydir = (ydiff >= 0) ? 1 : -1;

	bool avoidPlayer = (other->isAttacking() && other->getInfo()->anims[other->getInfo()->anim].breakthrough && xdist < 64);
	if (avoidPlayer && player->cpu.guardTick > 10)
		player->cpu.guardTick = rand() % 10;

	if (player->getStun() > 10)
	{
		player->cpu.guardTick = 15 + (rand() % 60);
		player->Inputs()->attack = false;
	}
	else if (player->getGuardStun() > 0)
	{
		player->cpu.guardTick = 10 + (rand() % 20);
		player->Inputs()->attack = false;
	}
	else if (player->cpu.guardTick > 0)
	{
		player->cpu.guardTick--;
		player->Inputs()->attack = false;
	}
	else
	{
		player->Inputs()->dirX = (avoidPlayer) ? -xdir : (xdist > 16 || player->getDir() != xdir) ? xdir : 0;
	}

	player->cpu.tacticSwitch--;
	if (player->cpu.tacticSwitch <= 0)
	{
		player->cpu.tactic = rand() % 7;
		player->cpu.tacticSwitch = 60 + (rand() % 30);
	}

	FighterAnim* anim = &player->getInfo()->anims[player->getInfo()->anim];
	player->Inputs()->jump = false;
	if (player->cpu.tactic == 0 && xdist <= 40 && ydist < 32) // jab combo
	{
		player->Inputs()->attack = true;
		player->Inputs()->dirY = 0;
		player->Inputs()->trigger = false;
		//if (player->cpu.tacticSwitch > 0) player->cpu.tacticSwitch--;
	}
	else if (player->cpu.tactic == 1 && xdist <= 40 && ydist < 32) // jab & sweep
	{
		player->Inputs()->attack = true;
		player->Inputs()->trigger = false;
		//player->Inputs()->dirY = (player->getInfo()->anim == "jabcombo2" && player->getInfo()->frame >= anim->combo.frame-1) ? 1 : 0;
		player->Inputs()->dirY = 1;
		u32 nextt = (player->getInfo()->anim == "sweepkick") ? rand() % 50 : 0;
		if (nextt >= 40)
		{
			player->cpu.tactic = 4; // spindash
			player->cpu.tacticSwitch = 90;
		}
		else if (nextt >= 30)
		{
			player->cpu.tactic = 3; // bounce
			player->cpu.tacticSwitch = 60;
		}
		else if (nextt >= 15)
		{
			player->Inputs()->jump = true;
		}
	}
	else if (player->cpu.tactic == 2 || (xdist < 32 && ydist < 32)) // up kick
	{
		player->Inputs()->dirY = -1;
		player->Inputs()->trigger = false;
		player->Inputs()->attack = true;
		if (rand() % 50 >= 40)
		{
			player->cpu.tactic = rand() % 7;
			player->cpu.tacticSwitch = 90;
		}
	}
	else if (player->cpu.tactic == 3 && xdist <= 40 && ydist < 56) // bounce
	{
		player->Inputs()->dirY = -1;
		player->Inputs()->trigger = true;
		player->Inputs()->attack = true;
	}
	else if (player->cpu.tactic == 4 && xdist < 256) // spin dash
	{
		player->Inputs()->dirY = 0;
		player->Inputs()->trigger = true;
		player->Inputs()->attack = true;
		player->Inputs()->attackHold = true;
		player->cpu.tacticSwitch++; // don't run out

		if (player->cpu.releaseAttack <= 0) player->cpu.releaseAttack = rand() % 67;

		player->cpu.releaseAttack--;
		if (player->cpu.releaseAttack <= 0)
		{
			player->Inputs()->attack = false;
			player->Inputs()->attackHold = false;
			player->cpu.releaseAttack = false;
			player->cpu.tactic = rand() % 7;
			player->cpu.tacticSwitch = 90;
		}
	}
	else if (player->cpu.tactic == 5 && xdist < 48) // flash kick
	{
		player->Inputs()->dirY = 1;
		player->Inputs()->trigger = true;
		player->Inputs()->attack = true;
		player->cpu.tactic = 0;
		player->cpu.tacticSwitch = 100;
	}
	else if (player->cpu.tactic == 6 && xdist < 128) // dive kick
	{
		player->Inputs()->jump = true;
		player->Inputs()->trigger = false;
		player->Inputs()->attack = false;
		if (ydir > 0 && xdist < 100)
		{
			player->Inputs()->dirY = 1;
			player->Inputs()->trigger = true;
			player->Inputs()->attack = true;
			if (rand() % 30 >= 20)
			{
				player->cpu.tactic = rand() % 7;
				player->cpu.tacticSwitch = 90;
			}
		}
	}
	else
	{
		player->Inputs()->attack = false;
		player->Inputs()->trigger = false;
	}

	if (player->Inputs()->attack)
	{
		if (player->cpu.attackTick <= 0) player->cpu.attackTick = 8;
		else
		{
			player->cpu.attackTick--;
			player->Inputs()->attack = false;
		}
	}
	else
		player->cpu.attackTick = 8;

	player->Inputs()->guard = (player->cpu.guardTick > 0);
}

// action callbacks END



void loadActions()
{
	if (!actionfuncs.empty()) return;

	actionfuncs["Mario_guard"] = genericGuard;
	actionfuncs["Mario_toFalling"] = genericToFalling;
	actionfuncs["Mario_hurt"] = genericHurt;
	actionfuncs["Mario_toReeling"] = genericToReeling;
	actionfuncs["Mario_reeling"] = genericReeling;
	actionfuncs["Mario_resetRotation"] = genericResetRotation;
	actionfuncs["Mario_stopAttacking"] = genericStopAttacking;
	actionfuncs["Mario_missB"] = genericMissB;
	actionfuncs["Mario_jump"] = marioJump;
	actionfuncs["Mario_painful_landing"] = marioPainfulLanding;
	actionfuncs["Mario_jabcombo"] = marioJabCombo1;
	actionfuncs["Mario_jabcombo2"] = marioJabCombo2;
	actionfuncs["Mario_jabcombo3"] = marioJabCombo3;
	actionfuncs["Mario_uppercut"] = marioUppercut;
	actionfuncs["Mario_hookpunch"] = marioHookPunch;
	actionfuncs["Mario_smackdown"] = marioSmackDown;
	actionfuncs["Mario_airuppercut"] = marioAirUppercut;
	actionfuncs["Mario_eaglekick"] = marioEaglekick;
	actionfuncs["Mario_groundpound1"] = marioPreGroundpound;
	actionfuncs["Mario_groundpound2"] = marioGroundpound;
	actionfuncs["Mario_groundpoundToLand"] = marioGroundpoundToLand;
	actionfuncs["Mario_groundpoundLand"] = marioGroundpoundLand;
	actionfuncs["Mario_sjp1"] = marioSjp1;
	actionfuncs["Mario_sjp2"] = marioSjp2;
	actionfuncs["Mario_sjp3"] = marioSjp3;
	actionfuncs["Mario_sjp4"] = marioSjp4;
	actionfuncs["Mario_hammerhop0"] = marioHammerhop0;
	actionfuncs["Mario_hammerhop1"] = marioHammerhop1;
	actionfuncs["Mario_hammerhop2"] = marioHammerhop2;
	actionfuncs["Mario_sndstrain"] = marioSndStrain;
	actionfuncs["Mario_hammersmash1"] = marioHammerSmash1;
	actionfuncs["Mario_hammersmash2"] = marioHammerSmash2;
	actionfuncs["Mario_hammersmash3"] = marioHammerSmash3;
	actionfuncs["Mario_hammersmash4"] = marioHammerSmash4;
	actionfuncs["Mario_hammersmash5"] = marioHammerSmash5;
	actionfuncs["Mario_Prehammermeteor"] = marioPreHammerMeteor;
	actionfuncs["Mario_hammermeteor"] = marioHammerMeteor;
	actionfuncs["Mario_Posthammermeteor"] = marioPostHammerMeteor;
	actionfuncs["Mario_hammermeteorLand"] = marioHammerMeteorLand;
	actionfuncs["MarioCPU"] = marioCPU;

	actionfuncs["Sonic_guard"] = genericGuard;
	actionfuncs["Sonic_toFalling"] = genericToFalling;
	actionfuncs["Sonic_hurt"] = genericHurt;
	actionfuncs["Sonic_toReeling"] = genericToReeling;
	actionfuncs["Sonic_reeling"] = genericReeling;
	actionfuncs["Sonic_airrecover"] = genericAirRecover;
	actionfuncs["Sonic_toFallNow"] = genericToFallNow;
	actionfuncs["Sonic_missLight"] = genericMissLight;
	actionfuncs["Sonic_missB"] = genericMissB;
	actionfuncs["Sonic_swing"] = genericSwing;
	actionfuncs["Sonic_stopAttacking"] = genericStopAttacking;
	actionfuncs["Sonic_stopspin"] = genericStopSpin;
	actionfuncs["Sonic_jump"] = sonicJump;
	actionfuncs["Sonic_painful_landing"] = sonicPainfulLanding;
	actionfuncs["Sonic_jabcombo1"] = sonicJabCombo1;
	actionfuncs["Sonic_jabcombo2a"] = sonicJabCombo2a;
	actionfuncs["Sonic_jabcombo2b"] = sonicJabCombo2b;
	actionfuncs["Sonic_jabcombo3a"] = sonicJabCombo3a;
	actionfuncs["Sonic_jabcombo3b"] = sonicJabCombo3b;
	actionfuncs["Sonic_sweepkick1"] = sonicSweepKick1;
	actionfuncs["Sonic_sweepkick2"] = sonicSweepKick2;
	actionfuncs["Sonic_sweepkick_jump"] = sonicSweepKickJump;
	actionfuncs["Sonic_flipkick1"] = sonicFlipKick1;
	actionfuncs["Sonic_flipkick2"] = sonicFlipKick2;
	actionfuncs["Sonic_tornadokick1"] = sonicTornadoKick1;
	actionfuncs["Sonic_tornadokick2"] = sonicTornadoKick2;
	actionfuncs["Sonic_downkick1"] = sonicDownKick1;
	actionfuncs["Sonic_downkick2"] = sonicDownKick2;
	actionfuncs["Sonic_spinsound"] = sonicSpinSound;
	actionfuncs["Sonic_stopdashing"] = sonicStopDashing;
	actionfuncs["Sonic_spindash_slide1"] = sonicSpinDashSlide1;
	actionfuncs["Sonic_spindash_slide2"] = sonicSpinDashSlide2;
	actionfuncs["Sonic_spindash_boost1"] = sonicSpinDashBoost1;
	actionfuncs["Sonic_spindash_boost2"] = sonicSpinDashBoost2;
	actionfuncs["Sonic_flashkick1"] = sonicFlashKick1;
	actionfuncs["Sonic_flashkick2"] = sonicFlashKick2;
	actionfuncs["Sonic_flashkick3"] = sonicFlashKick3;
	actionfuncs["Sonic_bounce"] = sonicBounce;
	actionfuncs["Sonic_toJump2"] = sonicToJump2;
	actionfuncs["Sonic_bounceair"] = sonicBounceAir;
	actionfuncs["Sonic_soniceagle1"] = sonicEagle1;
	actionfuncs["Sonic_predivekick"] = sonicPreDiveKick;
	actionfuncs["Sonic_divekickStart"] = sonicDiveKickStart;
	actionfuncs["Sonic_divekickMove"] = sonicDiveKickMove;
	actionfuncs["SonicCPU"] = sonicCPU;
}

ActionCallback getAction(std::string name)
{
	return (actionfuncs.count(name)) ? actionfuncs[name] : NULL;
}