/*
    Copyright 2008-2013 Risike, Reylak, MicroLua's developers

    MicroCPP by Headshotnoby, 2021-2022
*/

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <unistd.h>

//extern "C" {
#include <nds.h>
#include <filesystem.h>
#include <ulib/ulib.h>
#include <fat.h>
#include <mp3_shared.h>
//}

#include <microcpp/microcpp.h>

#include "main.h"
#include "engine.h"
#include "mainmenu.h"
#include "ui_title.h"
#include "ui_charselect.h"

Mainmenu* mainmenu;
Engine* engine;

int main()
{
	setup_uLibrary(false); // first argument is whether you want to use dual screen mode or not

	// your project below this line!

	mainmenu = new Mainmenu;
	engine = new Engine;

	mainmenu->init(new TitleUI, true);

	while (true)
	{
		while (mainmenu->isStarted())
		{
			Controls::read();
			mainmenu->tick();
			mainmenu->draw();
			screen::render();
		}

		mp3_stop();
		engine->setStage(mainmenu->selectedStage);
		for (int i=0; i<2; i++) engine->setFighter(i, mainmenu->selectedChars[i], (i==0));
		engine->start();

		while (engine->isStarted())
		{
			Controls::read();
			engine->tick();
			engine->draw();
			screen::render();
		}

		mainmenu->init(new CharSelectUI, true);
	}

	/*
	UL_FONT* font = Font::load("font.oft");
	UL_IMAGE* image = Image::load("lua/scripts/smbz-ds/data/stages/yoshi_ocean/sky.png", TO_RAM);
	std::string jm = "yoshi_ocean";
	char path[128], keys[128];

	rapidjson::Document document;
	document.Parse(json);

	sprintf(path, "lua/scripts/smbz-ds/data/stages/%s/intro.mp3", jm.c_str());
	mp3_play(path);

	Timer* timer = new Timer;
	timer->start();

	while (true)
	{
		if (!mp3_is_playing())
		{
			sprintf(path, "lua/scripts/smbz-ds/data/stages/%s/loop.mp3", jm.c_str());
			mp3_play(path, 1);
		}

		Controls::read();
		sprintf(keys, "nou: %s, %f", , timer->getTime()/1000);

		screen::blit(SCREEN_UP, 0, 0, image);
		screen::printFont(SCREEN_UP, 8, 8, "Hello world!", font);
		screen::print(SCREEN_DOWN, 8, 8, "Drawing from MicroCPP");
		screen::print(SCREEN_DOWN, 8, 192-16, keys);

		screen::render();
	}*/

	return 0;
}

void gameCrash(const char* msg)
{
	mp3_stop();
	if (mainmenu) mainmenu->deinit();
	if (engine) engine->stop();

	UL_IMAGE* img = Image::load("/data/smbz-ds/menu/crash.png", TO_RAM);
	std::string text1 = "SMBZ-DS has crashed!";
	std::string text2 = "Please report this on the GBAtemp post";
	std::string text3 = "if you think this was unexpected.";
	Image::setTint(img, RGB15(16,16,16));

	while (true)
	{
		screen::blit(SCREEN_UP, 0, 0, img);
		screen::print(SCREEN_UP, 128-(text1.size()*6/2)+1, 8+1, text1.c_str(), 0);
		screen::print(SCREEN_UP, 128-(text1.size()*6/2), 8, text1.c_str(), RGB15(31,0,0));
		screen::print(SCREEN_UP, 128-(text2.size()*6/2)+1, 20+1, text2.c_str(), 0);
		screen::print(SCREEN_UP, 128-(text2.size()*6/2), 20, text2.c_str());
		screen::print(SCREEN_UP, 128-(text3.size()*6/2)+1, 28+1, text3.c_str(), 0);
		screen::print(SCREEN_UP, 128-(text3.size()*6/2), 28, text3.c_str());

		screen::drawTextBox(SCREEN_UP, 4+1, 48+1, 256-4, 192, msg, 0);
		screen::drawTextBox(SCREEN_UP, 4, 48, 256-4, 192, msg);
		screen::render();
	}
}


void print_error(const char *text)
{
	while (1)
	{
		ulStartDrawing2D();
		ulDrawTextBox(0,0,256,192,text,0);
		ulEndDrawing();
		ulSyncFrame();
	}
}

void setup_uLibrary(bool dual)
{
	defaultExceptionHandler();
	mp3_init();
	screen::init();

    // Initialization of library
    ulInit(UL_INIT_ALL);
    ulInitGfx();
    ulInitText();

	int banks = UL_BANK_A | UL_BANK_B;
	int banksize = 256;
	if (dual)
    {
		// Sucks banks C-D + drops at 30 fps
		dualScreen = true;
		ulInitDualScreenMode();
	}
	else
	{
		videoSetModeSub(MODE_0_2D); // give sub screen a black background
		banks |= UL_BANK_C | UL_BANK_D;
		banksize += 256;
	}

    ulSetTexVramParameters(banks, VRAM_A, (banksize) << 10);

    // Banks F-G for palettes VRAM
    //ulSetTexPalVramParameters(UL_BANK_F | UL_BANK_E | UL_BANK_G | UL_BANK_H | UL_BANK_I, VRAM_E, (64+16+16+32+16) << 10); // this line causes visual glitches on NO$GBA and black screen on DraStic
	//ulSetTexPalVramParameters(UL_BANK_F | UL_BANK_G | UL_BANK_H | UL_BANK_I, VRAM_F, (64+16) << 10);

    // Use bright pink as a transparent color
    ulSetTransparentColor(RGB15(31, 0, 31));

#ifdef MICROLUA_NITROFS
	if (!nitroFSInit())
	{
		print_error("Failed to initialize NitroFS\nMake sure the nitrofiles folder exists in the project folder");
		return;
	}
    chdir("/");
#else
	if (!fatInitDefault())
	{
		print_error("Failed to initialize LibFat");
		return;
	}
#endif

    timerStart(TIMER_ID, ClockDivider_1024, 0, NULL);
}
