#include <microcpp/microcpp.h>
#include "mainmenu.h"
#include "ui.h"

void pause(const char* text)
{
	for (int i=0; i<60; i++)
	{
		screen::print(0, 8, 8, text);
		screen::render();
	}
}

void Mainmenu::init(UI* ui, bool firsttime)
{
	fadeAlpha = 100;
	fadeAdd = 0;
	logoX = 128;
	logoScale = 10;

	if (charList.empty()) charList = System::listDirectory("/data/smbz-ds/fighters", true);
	if (stageList.empty()) stageList = System::listDirectory("/data/smbz-ds/stages", true);

	mp3_set_volume(64);
	imgField = Image::load("/data/smbz-ds/menu/field.png", TO_VRAM);
	logo = Image::load("/data/smbz-ds/menu/smbzlogo.png", TO_VRAM);
	button = Image::load("/data/smbz-ds/menu/button.png", TO_VRAM);
	titleselect = wav_load_handle("/data/smbz-ds/menu/title_select.wav");
	sndselect = wav_load_handle("/data/smbz-ds/menu/select.wav");

	switchUI(ui, true, firsttime);

	started = true;
}

void Mainmenu::deinit()
{
	destroy();
	Image::destroy(imgField);
	Image::destroy(logo);
	Image::destroy(button);
	wav_free_handle(titleselect);
	wav_free_handle(sndselect);
	imgField = logo = button = NULL;
	titleselect = sndselect = NULL;

	started = false;
}

void Mainmenu::destroy()
{
	if (m_ui)
	{
		m_ui->destroy();
		delete m_ui;
		m_ui = NULL;
	}
}

void Mainmenu::switchUI(UI* which, bool fade, bool firsttime)
{
	if (fade)
	{
		switchTo = which;
		switchFirstTime = firsttime;
		fadeAdd = 2;
	}
	else
	{
		destroy();
		m_ui = which;

		if (m_ui)
			m_ui->init(this, firsttime);
		else // NULL, take it as "exit main menu"
			deinit();
	}
}

void Mainmenu::tick()
{
	fadeAlpha += fadeAdd;
	if (fadeAlpha > 100)
	{
		fadeAlpha = 100;
		fadeAdd = -2;
		switchUI(switchTo, false, switchFirstTime);
	}
	else if (fadeAlpha < 0)
	{
		fadeAlpha = 0;
		fadeAdd = 0;
	}

	if (m_ui) m_ui->tick();
}

void Mainmenu::draw()
{
	if (m_ui) m_ui->draw();

	if (fadeAlpha > 0)
	{
		screen::setAlpha(fadeAlpha);
		screen::drawFillRect(0, 0,0,256,192, 0);
		screen::setAlpha(100);
	}
}