#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <ulib/ulib.h>

#include <microcpp/color.h>
#include <microcpp/vars.h>

float hue2rgb(float p, float q, float t)
{
	if (t < 0) t++;
	if (t > 1) t--;
	if (t < 1./6) return p + (q - p) * 6 * t;
	if (t < 1./2) return q;
	if (t < 2./3) return p + (q - p) * (2./3 - t) * 6;
	return p;
}

namespace Color
{
	int new256(int r, int g, int b)
	{
		r = floor((float)r * 31 / 255);
		g = floor((float)g * 31 / 255);
		b = floor((float)b * 31 / 255);
		return RGB15(r, g, b);
	}

	int newHSL(float h, float s, float l)
	{
		int r, g, b;

		if (s == 0)
		{
			// achromatic
			r = l * 31;
			g = l * 31;
			b = l * 31;
		}
		else
		{
			float q = (l < 0.5) ? (l * (1 + s)) : (l + s - l * s);
			float p = 2 * l - q;
			
			r = hue2rgb(p, q, h + 1./3) * 31;
			g = hue2rgb(p, q, h) * 31;
			b = hue2rgb(p, q, h - 1./3) * 31;
		}

		return RGB15(r, g, b);
	}
}
