#include <fstream>
#include <memory>
#include <math.h>

#include <microcpp/microcpp.h>
#include <rapidjson/document.h>
#include <rapidjson/istreamwrapper.h>
#include <rapidjson/error/en.h>

#include <player.h>
#include <fighterinfo.h>
#include <engine.h>
#include <main.h>

void pause3(const char* text)
{
	for (int i=0; i<60; i++)
	{
		screen::print(0, 8, 8, text);
		screen::render();
	}
}

void Rotate(float &x, float &y, float cx, float cy, float angle)
{
	float s = sin(angle);
	float c = cos(angle);

	x -= cx;
	y -= cy;

	float xnew = (x * c - y * s);
	float ynew = (x * s + y * c);

	x = xnew + cx;
	y = ynew + cy;
}

bool aabb(float x0, float y0, float w0, float h0, float x1, float y1, float w1, float h1)
{
	return (x0 < x1 + w1 &&
			x0 + w0 > x1 &&
			y0 < y1 + h1 &&
			y0 + h0 > y1);
}

bool drawhitbox = false;


void KOCallback(void* pUserData)
{
	Engine* engine = (Engine*)pUserData;
	engine->darkbgAdd = -2;
}

void Player::setFighter(std::string name, bool _isLocal)
{
	for (std::map<std::string, FighterAnim>::iterator i = info.anims.begin(); i != info.anims.end(); i++)
	{
		for (u32 j=0; j<i->second.frames.size(); j++)
		{
			//if (i->second.frames[j].onStart.get())
			//if (i->second.frames[j].onLoop.get())
		}
	}

	info.sheets.clear();
	info.sounds.clear();
	info.attacks.clear();
	info.anims.clear();
	info.name = name;
	isLocal = _isLocal;

	char* buf = new char[65536];

	sprintf(buf, "/data/smbz-ds/fighters/%s/fighter.json", name.c_str());
	rapidjson::Document doc;
	std::ifstream ifs(buf);
	rapidjson::IStreamWrapper isw(ifs);
	rapidjson::ParseResult ok = doc.ParseStream(isw);
	ifs.close();

	if (!ok)
	{
		sprintf(buf, "%s JSON parse error: %s (%u)", name.c_str(), rapidjson::GetParseError_En(ok.Code()), ok.Offset());
		gameCrash(buf);
	}

	info.dash = doc["dash"].GetString();
	info.speed = doc["speed"].GetFloat();
	for (u32 i=0; i<doc["sheets"].Capacity(); i++) info.sheets.push_back(doc["sheets"][i].GetString());
	for (u32 i=0; i<doc["sounds"].Capacity(); i++) info.sounds.push_back(doc["sounds"][i].GetString());
	for (rapidjson::Value::ConstMemberIterator i=doc["attacks"].MemberBegin(); i != doc["attacks"].MemberEnd(); i++)
	{
		info.attacks[i->name.GetString()] = i->value.GetString();
	}

	for (rapidjson::Value::ConstMemberIterator i=doc["anims"].MemberBegin(); i != doc["anims"].MemberEnd(); i++)
	{
		FighterAnim anim;

		anim.loop = (!i->value.HasMember("loop") || i->value["loop"].GetBool());
		anim.breakthrough = (i->value.HasMember("breakthrough") && i->value["breakthrough"].GetBool());
		anim.img = (i->value.HasMember("img")) ? i->value["img"].GetString() : info.sheets[0];
		anim.combo.on = i->value.HasMember("combo");
		if (anim.combo.on)
		{
			if (i->value["combo"].HasMember("anim")) anim.combo.nextAnim = i->value["combo"]["anim"].GetString();
			anim.combo.frame = i->value["combo"]["frame"].GetInt();
		}

		for (u32 j=0; j<i->value["frames"].Capacity(); j++)
		{
			FighterFrame frame;

			frame.x = i->value["frames"][j]["x"].GetInt();
			frame.y = i->value["frames"][j]["y"].GetInt();
			frame.w = i->value["frames"][j]["w"].GetInt();
			frame.h = i->value["frames"][j]["h"].GetInt();
			frame.delay = i->value["frames"][j]["delay"].GetInt();
			frame.rotate = (i->value["frames"][j].HasMember("rotate")) ? i->value["frames"][j]["rotate"].GetInt() : 0;
			frame.ox[0] = (i->value["frames"][j].HasMember("ox")) ? i->value["frames"][j]["ox"][0].GetInt() : 0;
			frame.ox[1] = (i->value["frames"][j].HasMember("ox")) ? i->value["frames"][j]["ox"][1].GetInt() : 0;
			frame.oy = (i->value["frames"][j].HasMember("oy")) ? i->value["frames"][j]["oy"].GetInt() : 0;
			frame.scaleX = (i->value["frames"][j].HasMember("scale")) ? i->value["frames"][j]["scale"][0].GetInt() : 1;
			frame.scaleY = (i->value["frames"][j].HasMember("scale")) ? i->value["frames"][j]["scale"][1].GetInt() : 1;

			frame.onStart = (i->value["frames"][j].HasMember("onStart")) ? getAction(name + "_" + i->value["frames"][j]["onStart"].GetString()) : NULL;
			frame.onLoop = (i->value["frames"][j].HasMember("onLoop")) ? getAction(name + "_" + i->value["frames"][j]["onLoop"].GetString()) : NULL;

			anim.frames.push_back(frame);
		}

		info.anims[i->name.GetString()] = anim;
	}

	cpu.callback = getAction(name + "CPU");

	delete[] buf;
}

void Player::init(int hp, bool reuse)
{
	m_hp = m_maxhp = hp;
	m_reuse = reuse;

	posX = posY = 0;
	velX = velY = 0;

	enabled = true;
	timeScale = 1;
	timeAutoReset = 0;

	rot = 0;

	for (u32 i=0; i<info.sheets.size(); i++)
	{
		char buf[256];
		sprintf(buf, "/data/smbz-ds/fighters/%s/%s.png", info.name.c_str(), info.sheets[i].c_str());
		sheets[info.sheets[i]] = (!reuse) ? Image::load(buf, TO_VRAM) : m_engine->getPlayer(0)->getSheet(info.sheets[i]);
	}
	for(u32 i=0; i<info.sounds.size(); i++)
	{
		char buf[256];
		sprintf(buf, "/data/smbz-ds/fighters/%s/%s.wav", info.name.c_str(), info.sounds[i].c_str());
		sounds[info.sounds[i]] = wav_load_handle(buf);
	}

	dir = 1;
	inputs.dirX = 0;
	inputs.dirY = 0;
	inputs.jump = false;
	inputs.guard = false;
	inputs.attack = false;
	inputs.attackHold = false;
	inputs.trigger = false;
	inputs.attackTicks = 0;
	inputs.attackType = false;

	attacking = false;
	guarding = false;
	stun = 0;
	guardStun = 0;
	hitsTo = 0;
	hitsFrom = 0;

	setAnim("idle");
	attackbox.on = false;

	cpu.enemy = NULL;
	cpu.guardTick = 0;
	cpu.tactic = 0;
	cpu.tacticState = 0;
	cpu.tacticSwitch = 1;
	cpu.attackTick = 8;
	cpu.releaseAttack = 0;
}

void Player::destroy()
{
	if (!m_reuse)
	{
		for (std::map<std::string, UL_IMAGE*>::iterator i = sheets.begin(); i != sheets.end(); i++)
		{
			Image::destroy(i->second);
		}
	}
	for (std::map<std::string, volatile wav_stuffs*>::iterator i = sounds.begin(); i != sounds.end(); i++)
	{
		wav_free_handle(i->second);
	}
	sheets.clear();
	sounds.clear();
	enabled = false;
}

void Player::tick(float dt)
{
	if (!enabled) return;

	if (m_hp <= 0) stun = 100; // don't unstun K.O.'d players
	dt *= timeScale;

	FighterAnim* anim = &info.anims[info.anim];
	FighterFrame* frame = &anim->frames[info.frame];
	float halfw = frame->w/2;
	float ox = halfw * frame->scaleX;
	float oy = (frame->h * frame->scaleY) - frame->oy;

	float boxX = posX - ox - frame->ox[0];
	float boxY = posY - oy;
	float boxW = frame->w;
	float boxH = frame->h;

	// get inputs
	if (m_engine->fightStarted())
	{
		if (isLocal) // you
		{
			inputs.dirX = (Key.held.Left) ? -1 : (Key.held.Right) ? 1 : 0;
			inputs.dirY = (Key.held.Up) ? -1 : (Key.held.Down) ? 1 : 0;
			inputs.jump = Key.newPress.A;
			inputs.guard = Key.held.X;
			inputs.attack = (Key.newPress.B || Key.newPress.Y || inputs.attackTicks > 0);
			inputs.attackHold = (Key.held.B || Key.held.Y);
			inputs.trigger = (Key.held.L || Key.held.R || Key.newPress.Y || (inputs.attackTicks > 0 && inputs.attackType));

			if (inputs.attack && inputs.attackTicks <= 0)
			{
				inputs.attackTicks = 5;
				inputs.attackType = inputs.trigger;
			}
			else if (inputs.attackTicks > 0)
				inputs.attackTicks--;
		}
		else if (cpu.callback) // cpu
		{
			if (!cpu.enemy)
			{
				for (int i=0; i<2; i++)
				{
					if (m_engine->getPlayer(i) != this)
					{
						cpu.enemy = m_engine->getPlayer(i);
						break;
					}
				}
			}
			cpu.callback(m_engine, this, cpu.enemy, false);
		}
	}
	else
	{
		bool attack = (Key.newPress.B || Key.newPress.Y || inputs.attackTicks > 0);
		bool trigger = (Key.held.L || Key.held.R || Key.newPress.Y || (inputs.attackTicks > 0 && inputs.attackType));

		if (attack && inputs.attackTicks <= 0)
		{
			inputs.attackTicks = 20;
			inputs.attackType = trigger;
		}
		else if (inputs.attackTicks > 0)
			inputs.attackTicks--;
	}

	// process inputs and update states

	Player* other = m_engine->getPlayer(1-ind);
	if (other->attackBox()->on)
	{
		FighterAnim* otheranim = &other->getInfo()->anims[other->getInfo()->anim];
		FighterFrame* otherframe = &otheranim->frames[other->getInfo()->frame];
		float otherhalfw = otherframe->w/2;
		float otherox = otherhalfw * otherframe->scaleX;
		float otheroy = (otherframe->h * otherframe->scaleY) - otherframe->oy;

		float otherBoxX = (other->getDir() > 0) ?
			(other->getPosX() - otherox - otherframe->ox[0] + other->attackBox()->x) :
			(other->getPosX() + otherox + otherframe->ox[1] - other->attackBox()->x - other->attackBox()->w);
		float otherBoxY = other->getPosY() - otheroy + other->attackBox()->y;
		float otherBoxW = other->attackBox()->w;
		float otherBoxH = other->attackBox()->h;

		// check for collision!
		if (aabb(boxX, boxY, boxW, boxH, otherBoxX, otherBoxY, otherBoxW, otherBoxH))
		{
			// this player got hit!
			bool hitGuarded = (guarding && ((dir == -1 && other->getPosX() - posX < 0) || (dir == 1 && other->getPosX() - posX > 0)));
			other->attackBox()->on = false;
			if (other->attackBox()->onHit) other->attackBox()->onHit(m_engine, other, this, hitGuarded);

			// in case if the onHit callback changed the animation (99% sure it will though)
			anim = &info.anims[info.anim];
			frame = &anim->frames[info.frame];
		}
	}

	float spd = abs(velX);
	int spdsign = (velX > 0) ? 1 : (velX < 0) ? -1 : 0;
	bool jumping = (info.anim == "jump");
	bool can_move = (m_engine->fightStarted() && !inputs.guard && guardStun <= 0 && !attacking && stun <= 0);
	bool grounded = (posY >= 0);

	if (can_move)
	{
		velX += inputs.dirX*20;
		spd = abs(velX);
		if (spd > info.speed) // clamp
			velX = info.speed*dir;
	}
	else if (stun > 0)
	{
		stun -= dt;
		if (stun <= 0)
			hitsFrom = 0;
	}

	if (grounded)
	{
		posY = 0;
		if (velY > 0) velY = 0;

		if (inputs.jump && !jumping && can_move)
		{
			setAnim("jump");
			anim = &info.anims[info.anim];
			frame = &anim->frames[info.frame];
			jumping = true;
		}
		else if (inputs.dirX) // holding left/right key
		{
			if (stun <= 0 && !attacking) dir = inputs.dirX;
			if (!jumping && can_move)
			{
				setAnim((spd < info.speed) ? "walk" : "run");
				anim = &info.anims[info.anim];
				frame = &anim->frames[info.frame];
			}
		}
		else if (!jumping && !velX && can_move)
		{
			setAnim("idle");
			anim = &info.anims[info.anim];
			frame = &anim->frames[info.frame];
		}

		if (velX && (!inputs.dirX || !can_move)) // slide to a stop
		{
			if (!jumping && can_move)
			{
				setAnim("skid");
				anim = &info.anims[info.anim];
				frame = &anim->frames[info.frame];
			}
			float friction = (stun<=0 ? 15 : 5) * dt;
			//float before = (abs(velX)) * spdsign;
			float after = (abs(velX) - friction) * spdsign;
			int afterspdsign = (after > 0) ? 1 : (after < 0) ? -1 : 0;
			if (spdsign != afterspdsign) after = 0; // stopped
			velX = after;
		}
	}
	else // in the air
	{
		velY += 20 * dt; // gravity
		if (inputs.dirX && stun <= 0 && !attacking)
			dir = inputs.dirX;
	}

	// execute attack
	if (stun <= 0 && inputs.attack)
	{
		std::string key;
		key += (inputs.trigger) ? "z" : "";
		key += (inputs.guard) ? "guard" : "";
		key += (inputs.dirY < 0) ? "up" : (inputs.dirY > 0) ? "down" : "";
		key += (grounded) ? "neutral" : "air";

		/*
		all possible attack keys:
			neutral
			upneutral
			downneutral
			guardneutral
			guardupneutral
			guarddownneutral
			air
			upair
			downair
			guardair
			guardupair
			guarddownair
			
			prefix "z" on any of these for z-trigger attacks, e.g. "zneutral" or "zdownair"
		*/

		if (info.attacks.count(key))
		{
			std::string fightanim = info.attacks[key];

			if (!attacking || (attackName != key && (!anim->combo.on || info.frame >= anim->combo.frame-1)))
			{
				// launch attack
				attackName = key;
				attacking = true;
				setAnim(fightanim, true);
				anim = &info.anims[info.anim];
				frame = &anim->frames[info.frame];
			}
			else if (attacking && attackName == key && anim->combo.on && anim->combo.nextAnim.size() && info.frame >= anim->combo.frame-1)
			{
				// combo attack
				attacking = true;
				setAnim(anim->combo.nextAnim, true);
				anim = &info.anims[info.anim];
				frame = &anim->frames[info.frame];
			}
		}
	}

	guarding = (guardStun > 0 || (inputs.guard && stun <= 0 && !attacking));
	if (guarding)
	{
		setAnim("guard");
		anim = &info.anims[info.anim];
		frame = &anim->frames[info.frame];
		if (guardStun > 0) guardStun--;
	}


	// sprite animation


	info.delay += dt;
	if (frame->rotate) rot += (frame->rotate * -dir * dt);

	if (frame->onLoop)
	{
		frame->onLoop(m_engine, this, NULL, false);
	}

	float delay = frame->delay;
	while (info.delay >= delay)
	{
		info.delay -= delay;
		info.frame++;
		if (attackbox.deleteOnNextFrame) attackbox.on = false;

		if (info.frame >= anim->frames.size())
		{
			if (anim->loop) // loop the animation
			{
				info.frame = 0;
			}
			else
			{
				info.frame--;
				attacking = false;
			}
			info.loops++;
		}
		frame = &anim->frames[info.frame];

		if (frame->onStart)
		{
			frame->onStart(m_engine, this, NULL, false);
		}
	}


	// update position


	posX += velX / 100 * dt;
	posY += velY / 100 * dt;
	if (!grounded && posY > 0)
	{
		posY = 0;
		playSnd((stun <= 0) ? "landLight" : (velY > 350) ? "landHard" : "landMedium");
		if (velY > 250 && stun > 0)
			velY = -velY / 3.f; // bounce
	}
}

void Player::draw(float screenW, float screenH)
{
	float camX = m_engine->CamX();
	float camY = m_engine->CamY();
	float playerX = floor(posX - (camX - screenW/2));
	float playerY = floor(posY - (camY - screenH/2));

	FighterAnim* anim = &info.anims[info.anim];
	FighterFrame* frame = &anim->frames[info.frame];
	UL_IMAGE* img = sheets[anim->img];

	// Drawing order for a quad is: top-left, bottom-left, bottom-right, top-right.
	// reverse: top-right, bottom-right, bottom-left, top-left

	float x0,y0,x1,y1,x2,y2,x3,y3, ox, oy, fx;
	float halfw = frame->w/2;
	float halfh = frame->h/2;
	ox = halfw * frame->scaleX;
	oy = (frame->h * frame->scaleY) - frame->oy;
	char buf[128];

	if (dir > 0) // right
	{
		x0 = playerX;									y0 = playerY;
		x1 = playerX;									y1 = playerY + (frame->h*frame->scaleY);
		x2 = playerX + (frame->w*frame->scaleX);		y2 = playerY + (frame->h*frame->scaleY);
		x3 = playerX + (frame->w*frame->scaleX);		y3 = playerY;
		fx = frame->x;									ox += frame->ox[0];

		if (frame->rotate)
		{
			float Rot = rot/1000;
			Rotate(x0, y0, x0+halfw, y0+halfh, Rot);
			Rotate(x1, y1, x1+halfw, y1-halfh, Rot);
			Rotate(x2, y2, x2-halfw, y2-halfh, Rot);
			Rotate(x3, y3, x3-halfw, y3+halfh, Rot);
		}
	}
	else // left
	{
		x0 = playerX + (frame->w*frame->scaleX);		y0 = playerY+0.5f;
		x1 = playerX + (frame->w*frame->scaleX);		y1 = playerY + (frame->h*frame->scaleY);
		x2 = playerX;									y2 = playerY + (frame->h*frame->scaleY);
		x3 = playerX;									y3 = playerY+0.5f;
		fx = frame->x-1;								ox -= frame->ox[1];

		if (frame->rotate)
		{
			float Rot = rot/1000;
			Rotate(x0, y0, x0-halfw, y0+halfh, Rot);
			Rotate(x1, y1, x1-halfw, y1-halfh, Rot);
			Rotate(x2, y2, x2+halfw, y2-halfh, Rot);
			Rotate(x3, y3, x3+halfw, y3+halfh, Rot);
		}
	}

	// call the majestic function
	screen::drawTexturedQuad(SCREEN_UP,
							x0-ox, y0-oy,
							x1-ox, y1-oy,
							x2-ox, y2-oy,
							x3-ox, y3-oy,
							img,
							RGB15(31, 31, (stun > 0)?0:31),
							fx, frame->y,
							frame->w, frame->h);

	if (attackbox.on && drawhitbox)
	{
		float boxX = (dir > 0) ?
					 (playerX - (halfw * frame->scaleX) - frame->ox[0] + attackbox.x) :
					 (playerX + (halfw * frame->scaleX) + frame->ox[1] - attackbox.x - attackbox.w);
		float boxY = playerY - oy + attackbox.y;

		screen::setAlpha(50);
		screen::drawFillRect(SCREEN_UP, boxX, boxY, boxX+attackbox.w, boxY+attackbox.h, RGB15(31,0,0));
		screen::setAlpha(100);
	}

	sprintf(buf, "%d", ind+1);
	screen::print(SCREEN_UP, x0-ox-3, y0-oy, buf);
}

void Player::facePlayer(Player* who)
{
	float xdif = who->getPosX() - posX;
	dir = (xdif >= 0) ? 1 : -1;
}

void Player::giveDamage(float hp, int addStun, bool guard)
{
	if (!guard)
	{
		m_hp -= hp;
		setAnim("hurt", true);
		hitsFrom++;
		int div = hitsFrom/2;
		if (!div) div++;
		stun += addStun/div;

		attackbox.on = false;
		attacking = false;
		guarding = false;
		guardStun = 0;

		m_engine->moveToFront(ind);
	}
	else
		guardStun = 20;

	if ((m_hp <= 0 || floor(m_hp)+1 <= 0) && m_engine->gameOverTicks() == 60*5) // K.O.!
	{
		m_engine->darkbgAdd = 2;
		float slowtime = wav_get_length(sounds["ko"]);
		if (slowtime < 2) slowtime = 2;
		playSnd("ko");
		m_engine->setTimeScale(0.1, floor(slowtime*60), KOCallback);
	}
}

void Player::knockBack(float x, float y, Player* other, bool guard, bool absoluteVel)
{
	if (y > 0 && posY >= 0)
	{
		x = floor(x*1.2);
		y = floor(-y*1.3);
	}
	if (guard)
	{
		x = floor(x/2);
		y = floor(y/2);
	}
	velX = x * other->getDir() + (absoluteVel ? 0 : other->getVelX());
	velY = y;
	if (y < 0 && posY >= 0) posY = -1; // jump
}

void Player::setAttackBox(bool on, int x, int y, int w, int h, bool deleteOnNextFrame, ActionCallback onHit)
{
	attackbox.on = on;
	attackbox.x = x;
	attackbox.y = y;
	attackbox.w = w;
	attackbox.h = h;
	attackbox.deleteOnNextFrame = deleteOnNextFrame;
	attackbox.onHit = onHit;
}

void Player::setTimeScale(float scale, int autoreset)
{
	timeScale = scale;
	timeAutoReset = autoreset;
}

void Player::playSnd(std::string name)
{
	int channel = 3+ind;
	if (sounds.count(name)) wav_play_handle(sounds[name], channel);
	else if (m_engine->snds.count(name)) wav_play_handle(m_engine->snds[name], channel);
}

void Player::setAnim(std::string name, bool restart, bool dontDisableAttack)
{
	if (!info.anims.count(name) || (info.anim == name && !restart)) return;
	if (!dontDisableAttack) attackbox.on = false;

	info.anim = name;
	info.frame = 0;
	info.delay = 0;
	info.loops = 0;

	if (info.anims[info.anim].frames[info.frame].onStart)
	{
		info.anims[info.anim].frames[info.frame].onStart(m_engine, this, NULL, false);
	}
}
