#include <microcpp/microcpp.h>

#include "ui_charselect.h"
#include "ui_stageselect.h"
#include "ui_title.h"

void CharSelectUI::init(Mainmenu* mainmenu, bool firsttime)
{
	m_mainmenu = mainmenu;

	for (u32 i=0; i<mainmenu->charList.size(); i++)
	{
		char buf[256];
		sprintf(buf, "/data/smbz-ds/fighters/%s/portrait.png", mainmenu->charList[i].name.c_str());
		portraits.push_back(Image::load(buf, TO_VRAM));
	}

	currPlayer = 0;
	selectedChars[0] = 0;
	selectedChars[1] = 0;

	if (!mp3_is_playing()) mp3_play("/data/smbz-ds/menu/music.mp3", 1);
}

void CharSelectUI::destroy()
{
	for (u32 i=0; i<m_mainmenu->charList.size(); i++)
	{
		Image::destroy(portraits[i]);
	}
	portraits.clear();
}

void CharSelectUI::tick()
{
	scrollX -= 0.2;
	if (scrollX <= -m_mainmenu->imgField->sizeX) scrollX = 0;

	if (Key.newPress.Down)
		selectedChars[currPlayer] = (selectedChars[currPlayer]+1) % portraits.size();
	else if (Key.newPress.Up)
		selectedChars[currPlayer] = (selectedChars[currPlayer] > 0) ? selectedChars[currPlayer]-1 : portraits.size()-1;
	else if (Key.newPress.A)
	{
		wav_play_handle(m_mainmenu->sndselect, 2);
		m_mainmenu->selectedChars[currPlayer] = m_mainmenu->charList[selectedChars[currPlayer]].name;

		if (currPlayer == 1)
			m_mainmenu->switchUI(new StageSelectUI(scrollX));
		else
			currPlayer++;
	}
	else if (Key.newPress.B)
	{
		if (currPlayer > 0)
			currPlayer--;
		else
			m_mainmenu->switchUI(new TitleUI, true);
	}
}

void CharSelectUI::draw()
{
	for (int i=0; i<m_mainmenu->imgField->sizeX*2; i+=m_mainmenu->imgField->sizeX) screen::blit(0, i + scrollX, 0, m_mainmenu->imgField);

	float x0,y0,x1,y1,x2,y2,x3,y3,ox,oy;
	float w = m_mainmenu->logo->sizeX * 0.5f;
	float h = m_mainmenu->logo->sizeY * 0.5f;

	x0 = 0;				y0 = 0;
	x1 = 0;				y1 = h;
	x2 = w;				y2 = h;
	x3 = w;				y3 = 0;
	ox = 128-(w/2);		oy = 4;
	screen::drawTexturedQuad(0, x0+ox,y0+oy,x1+ox,y1+oy,x2+ox,y2+oy,x3+ox,y3+oy, m_mainmenu->logo);

	std::string txt = "Select your characters";
	screen::print(0, 128-(txt.size()*6/2)+1, oy+h+2+1, txt.c_str(), 0);
	screen::print(0, 128-(txt.size()*6/2), oy+h+2, txt.c_str(), RGB15(31,0,0));

	for (int i=0; i<=1; i++) // two players
	{
		x0 = 16+(i*128);
		y0 = 48+24;

		std::string txt2 = (i>0) ? "CPU" : "You";
		screen::print(0, x0+48-(txt2.size()*6/2)+1, y0-10+1, txt2.c_str(), 0);
		screen::print(0, x0+48-(txt2.size()*6/2), y0-10, txt2.c_str(), RGB15(31,0,0));

		for (u32 ii=1; ii<=m_mainmenu->charList.size(); ii++)
		{
			y0 = 48+(ii*24);
			int color = RGB15(31, 31, (currPlayer >= i && selectedChars[i] == ii-1) ? 0 : 31);

			screen::drawFillRect(SCREEN_UP, x0, y0, x0+96, y0+18, color);
			screen::drawRect(SCREEN_UP, x0, y0, x0+96, y0+18, 0);
			screen::blit(SCREEN_UP, x0, y0, portraits[ii-1]);
			screen::print(SCREEN_UP, x0+20, y0+9-4, m_mainmenu->charList[ii-1].name.c_str(), 0);
		}
	}
}