#include <microcpp/microcpp.h>

#include "ui_stageselect.h"
#include "ui_charselect.h"

void StageSelectUI::init(Mainmenu* mainmenu, bool firsttime)
{
	m_mainmenu = mainmenu;

	selected = 0;
	menuScroll = 0;

	if (!mp3_is_playing()) mp3_play("/data/smbz-ds/menu/music.mp3", 1);
}

void StageSelectUI::destroy()
{
	// nothing to free from memory lol
}

void StageSelectUI::tick()
{
	scrollX -= 0.2;
	if (scrollX <= -m_mainmenu->imgField->sizeX) scrollX = 0;

	if (Key.newPress.Down)
		selected = (selected+1) % m_mainmenu->stageList.size();
	else if (Key.newPress.Up)
	{
		selected--;
		if (selected < 0) selected = m_mainmenu->stageList.size()-1;
	}
	else if (Key.newPress.A)
	{
		wav_play_handle(m_mainmenu->sndselect, 2);
		m_mainmenu->selectedStage = m_mainmenu->stageList[selected].name;
		m_mainmenu->switchUI(NULL, true); // exit main menu state with a graceful fade out
	}
	else if (Key.newPress.B)
	{
		m_mainmenu->switchUI(new CharSelectUI(scrollX));
	}

	while (selected > menuScroll+7) menuScroll++;
	while (selected < menuScroll) menuScroll--;
}

void StageSelectUI::draw()
{
	for (int i=0; i<m_mainmenu->imgField->sizeX*2; i+=m_mainmenu->imgField->sizeX) screen::blit(0, i + scrollX, 0, m_mainmenu->imgField);

	float x0,y0,x1,y1,x2,y2,x3,y3,ox,oy;
	float w = m_mainmenu->logo->sizeX * 0.5f;
	float h = m_mainmenu->logo->sizeY * 0.5f;

	x0 = 0;				y0 = 0;
	x1 = 0;				y1 = h;
	x2 = w;				y2 = h;
	x3 = w;				y3 = 0;
	ox = 128-(w/2);		oy = 4;
	screen::drawTexturedQuad(0, x0+ox,y0+oy,x1+ox,y1+oy,x2+ox,y2+oy,x3+ox,y3+oy, m_mainmenu->logo);

	std::string txt = "Select a stage";
	screen::print(0, 128-(txt.size()*6/2)+1, oy+h+2+1, txt.c_str(), 0);
	screen::print(0, 128-(txt.size()*6/2), oy+h+2, txt.c_str(), RGB15(31,0,0));

	for (int i=0; i<8; i++)
	{
		int x = 16;
		int y = 56+(i*16);
		u32 stageind = (u32)(i+menuScroll);
		if (stageind < m_mainmenu->stageList.size())
		{
			int color = RGB15(31, 31, ((u32)selected == stageind) ? 0 : 31);

			screen::drawFillRect(SCREEN_UP, x, y, x+256-32, y+16, color);
			screen::drawRect(SCREEN_UP, x, y, x+256-32, y+16, 0);
			screen::print(SCREEN_UP, 128-(m_mainmenu->stageList[stageind].name.size()*6/2), y+8-4, m_mainmenu->stageList[stageind].name.c_str(), 0);
		}
	}

	if ((u32)(menuScroll)+7 < m_mainmenu->stageList.size()-1)
	{
		x0 = -4;		y0 = -8;
		x1 = 0;			y1 = 0;
		x2 = 4;			y2 = -8;
		ox = 256-8;		oy = 56+(8*16)-4;
		screen::drawTriangle(SCREEN_UP, x0+ox-1, y0+oy-1, x1+ox, y1+oy+1, x2+ox+1, y2+oy-1, 0,0,0);
		screen::drawTriangle(SCREEN_UP, x0+ox, y0+oy, x1+ox, y1+oy, x2+ox, y2+oy, RGB15(31,0,0), RGB15(31,0,0), RGB15(31,0,0));
	}
	if (menuScroll > 0)
	{
		x0 = 0;			y0 = 0;
		x1 = -4;		y1 = 8;
		x2 = 4;			y2 = 8;
		ox = 256-8;		oy = 56+4;
		screen::drawTriangle(SCREEN_UP, x0+ox, y0+oy-1, x1+ox-1, y1+oy+1, x2+ox+1, y2+oy+1, 0,0,0);
		screen::drawTriangle(SCREEN_UP, x0+ox, y0+oy, x1+ox, y1+oy, x2+ox, y2+oy, RGB15(31,0,0), RGB15(31,0,0), RGB15(31,0,0));
	}
}