#include <fstream>
#include <math.h>

#include <microcpp/microcpp.h>
#include <rapidjson/document.h>
#include <rapidjson/istreamwrapper.h>

#include "ui_title.h"
#include "ui_charselect.h"

void pause5(const char* text)
{
	for (int i=0; i<60; i++)
	{
		screen::print(0, 8, 8, text);
		screen::render();
	}
}

void TitleUI::init(Mainmenu* mainmenu, bool firsttime)
{
	m_mainmenu = mainmenu;

	btn = 0;

	buttons[0].x = 128-64; buttons[0].y = m_mainmenu->logo->sizeY+32;
	buttons[1].x = 128+64; buttons[1].y = m_mainmenu->logo->sizeY+32;

	buttons[0].text = "Play";
	buttons[1].text = "Quit";

	buttons[0].setCallback(&onPlay, this);
	buttons[1].setCallback(&onQuit, this);

	for (int i=0; i<2; i++) scrollX[i] = 0;
	ground = Image::load("/data/smbz-ds/menu/ground.png", TO_VRAM);
	textZoom = 100;

	mp3_fill_buffer();

	char buf[1024];
	for (u32 i=0; i<mainmenu->charList.size(); i++)
	{
		sprintf(buf, "/data/smbz-ds/fighters/%s/title.png", mainmenu->charList[i].name.c_str());
		UL_IMAGE* sprite = Image::load(buf, TO_VRAM);

		sprintf(buf, "/data/smbz-ds/fighters/%s/title.json", mainmenu->charList[i].name.c_str());
		rapidjson::Document doc;
		std::ifstream ifs(buf);
		rapidjson::IStreamWrapper isw(ifs);
		doc.ParseStream(isw);
		ifs.close();

		spriteOffsets.push_back(doc["xoffset"].GetInt());
		CharInfo info;
		CharAnim anim;

		for (u32 j=0; j<doc["frames"].Capacity(); j++)
		{
			CharFrame frame;
			frame.x = doc["frames"][j]["x"].GetInt();
			frame.y = doc["frames"][j]["y"].GetInt();
			frame.w = doc["frames"][j]["w"].GetInt();
			frame.h = doc["frames"][j]["h"].GetInt();
			frame.ox = (doc["frames"][j].HasMember("ox")) ? doc["frames"][j]["ox"].GetInt() : 0;
			frame.delay = doc["frames"][j]["delay"].GetFloat();
			anim.frames.push_back(frame);
		}
		info.anims["title"] = anim;

		charSprites.push_back(std::pair<UL_IMAGE*, CharInfo>(sprite, info));

		mp3_fill_buffer();
	}

	if (firsttime)
	{
		currAnim = 0;
		mainmenu->logoX = -mainmenu->logo->sizeX;
		drawMenu = 0;
	}
	else
	{
		currAnim = 10;
		mainmenu->logoX = 128;
		drawMenu = 2;
	}

	if (!mp3_is_playing()) mp3_play("/data/smbz-ds/menu/music.mp3", 1);
}

void TitleUI::destroy()
{
	Image::destroy(ground);
	for (u32 i=0; i<charSprites.size(); i++)
	{
		Image::destroy(charSprites[i].first);
	}
	charSprites.clear();
	mp3_fill_buffer();
}

void TitleUI::tick()
{
	scrollX[0]--;
	scrollX[1] -= 37;
	if (scrollX[0] <= -m_mainmenu->imgField->sizeX) scrollX[0] = 0;
	if (scrollX[1] <= -ground->sizeX) scrollX[1] += ground->sizeX;

	fade += fadeAdd;
	if (fade > 100)
	{
		fade = 100;
		fadeAdd = 0;
	}
	else if (fade < 0)
	{
		fade = 0;
		fadeAdd = 0;
	}

	for (u32 i=0; i<charSprites.size(); i++)
	{
		CharInfo* info = &charSprites[i].second;
		CharFrame* frame = &info->anims["title"].frames[info->frame];

		info->delay += 1;
		if (info->delay >= frame->delay)
		{
			info->delay = 0;
			info->frame++;
			if (info->frame >= info->anims["title"].frames.size()) info->frame = 1;
		}
	}

	if (drawMenu == 2)
	{
		if (Key.newPress.Right)
			btn = (btn+1) % 2;
		else if (Key.newPress.Left)
			btn = abs((btn-1) % 2);
		else if (Key.newPress.A)
		{
			wav_play_handle(m_mainmenu->titleselect, 2);
			drawMenu = 1;
			buttons[btn].callback(buttons[btn].pUserData);
		}
	}

	if (Key.newPress.A && currAnim < 9) // skip animation
	{
		m_mainmenu->logoX = 128;
		currAnim = 9;

		wav_play_handle(m_mainmenu->titleselect, 2);
		fade = 100;
		fadeAdd = -2;
		drawMenu = 2;
		scaleAdd = -1;
	}

	switch (currAnim)
	{
		case 0:
			if (m_mainmenu->fadeAlpha <= 0)
			{
				currAnim++;
				textZoom = 60;
				fade = 0;
				fadeAdd = 2;
			}
			break;

		case 1:
			textZoom++;
			if (textZoom > 100) textZoom = 100;

			if (fade >= 100)
			{
				currAnim++;
				textZoom = 100;
				animTicks = 90;
			}
			break;

		case 2:
			animTicks--;
			if (animTicks < 0)
			{
				currAnim++;
				fadeAdd = -2;
			}
			break;

		case 3:
			if (fade <= 0)
			{
				currAnim++;
				textZoom = 60;
				fadeAdd = 2;
			}
			break;

		case 4:
			textZoom++;
			if (textZoom > 100) textZoom = 100;

			if (fade >= 100)
			{
				currAnim++;
				textZoom = 100;
				animTicks = 120;
			}
			break;

		case 5:
			animTicks--;
			if (animTicks < 0)
			{
				currAnim++;
				fadeAdd = -2;
			}
			break;

		case 6:
			if (fade <= 0)
			{
				currAnim++;
				animTicks = 30;
			}
			break;

		case 7:
			animTicks--;
			if (animTicks < 0)
				currAnim++;
			break;

		case 8:
			m_mainmenu->logoX += 16;
			if (m_mainmenu->logoX >= 128)
			{
				m_mainmenu->logoX = 128;
				currAnim++;

				wav_play_handle(m_mainmenu->titleselect, 2);
				fade = 100;
				fadeAdd = -2;
				drawMenu = 2;
				scaleAdd = -1;
			}
			break;

		case 9:
			fade += fadeAdd;
			if (fade <= 0)
			{
				fade = 0;
				fadeAdd = 0;
			}

			m_mainmenu->logoScale += scaleAdd;
			if (m_mainmenu->logoScale <= -10)
				scaleAdd = 1;
			else if (m_mainmenu->logoScale >= 10)
			{
				scaleAdd = 0;
				currAnim++;
			}
			break;
	}
}

void TitleUI::draw()
{
	for (int i=0; i<m_mainmenu->imgField->sizeX*2; i+=m_mainmenu->imgField->sizeX) screen::blit(0, i + scrollX[0], 0, m_mainmenu->imgField);
	for (int i=0; i<ground->sizeX*2; i+=ground->sizeX) screen::blit(0, i + scrollX[1], 192-ground->sizeY, ground);

	float x0,y0,x1,y1,x2,y2,x3,y3,ox,oy;
	float w = m_mainmenu->logo->sizeX * (m_mainmenu->logoScale/10);
	float h = m_mainmenu->logo->sizeY;

	x0 = 0;							y0 = 0;
	x1 = 0;							y1 = h;
	x2 = w;							y2 = h;
	x3 = w;							y3 = 0;
	ox = m_mainmenu->logoX-(w/2);	oy = 4;
	screen::drawTexturedQuad(0, x0+ox,y0+oy,x1+ox,y1+oy,x2+ox,y2+oy,x3+ox,y3+oy, m_mainmenu->logo);

	if (drawMenu > 0)
	{
		for (u32 i=0; i<charSprites.size(); i++)
		{
			CharInfo* info = &charSprites[i].second;
			CharFrame* frame = &info->anims["title"].frames[info->frame];

										y1 = frame->h;
			x2 = frame->w;				y2 = frame->h;
			x3 = frame->w;
			ox = 128.f-((float)(frame->w)/2)-frame->ox + spriteOffsets[i];		oy = 192-ground->sizeY+16 - frame->h;

			screen::drawTexturedQuad(0, x0+ox,y0+oy,x1+ox,y1+oy,x2+ox,y2+oy,x3+ox,y3+oy, charSprites[i].first, RGB15(31,31,31), frame->x, frame->y, frame->w, frame->h);
		}

		std::string txt1 = "v0.4";
		screen::print(SCREEN_UP, 128-((txt1.size()*6)/2), m_mainmenu->logo->sizeY+4, txt1.c_str(), 0);

		if (drawMenu == 2)
		{
			for (int i=0; i<2; i++)
			{
				float w = m_mainmenu->button->sizeX;
				float h = m_mainmenu->button->sizeY;
				float x = buttons[i].x - (w/2);
				float y = buttons[i].y;

				screen::blit(SCREEN_UP, x, y, m_mainmenu->button);
				screen::print(SCREEN_UP, x+(w/2)-(buttons[i].text.size()*6/2)+1, y+(h/2)-4+1, buttons[i].text.c_str(), 0);
				screen::print(SCREEN_UP, x+(w/2)-(buttons[i].text.size()*6/2), y+(h/2)-4, buttons[i].text.c_str(), RGB15(31,31,31));

				if (btn == i)
				{
					// Triangle order: top, bottom left, bottom right
					// draw an arrow of sorts pointing down
					x0 = -8;		y0 = -16;
					x1 = 0;			y1 = 0;
					x2 = 8;			y2 = -16;
					ox = x+(w/2)+1;	oy = y-4;
					screen::drawTriangle(SCREEN_UP, x0+ox-1, y0+oy-1, x1+ox, y1+oy+1, x2+ox+1, y2+oy-1, 0,0,0);
					screen::drawTriangle(SCREEN_UP, x0+ox,y0+oy,x1+ox,y1+oy,x2+ox,y2+oy, Color::new256(255, 180, 0), Color::new256(255, 180, 0), Color::new256(255, 180, 0));
				}
			}
		}
	}

	float screenW, screenH;
	std::string text[4] = {"Headshotnoby", "presents", "A DS fangame inspired by an animated", "Flash series by Alvin Earthworm"};
	switch (currAnim)
	{
		case 1:
		case 3:
			screenW = 256/(textZoom/100);
			screenH = 192/(textZoom/100);

			screen::scaleView(0, textZoom/100, textZoom/100);
			screen::setAlpha(fade);
			for (int i=1; i<=2; i++)
			{
				if (fade > 0)
				{
					if (fade < 100)
					{
						screen::print(0, screenW/2-(text[i-1].size()*6/2), screenH/2-4-(2*8)+(i*8), text[i-1].c_str());
						screen::print(0, screenW/2-(text[i-1].size()*6/2)+1, screenH/2-4-(2*8)+(i*8)+1, text[i-1].c_str(), 0);
					}
					else
					{
						screen::print(0, screenW/2-(text[i-1].size()*6/2)+1, screenH/2-4-(2*8)+(i*8)+1, text[i-1].c_str(), 0);
						screen::print(0, screenW/2-(text[i-1].size()*6/2), screenH/2-4-(2*8)+(i*8), text[i-1].c_str());
					}
				}
			}
			screen::resetView();
			screen::setAlpha(100);
			break;

		case 2:
			screenW = 256;
			screenH = 192;

			for (int i=1; i<=2; i++)
			{
				screen::print(0, screenW/2-(text[i-1].size()*6/2)+1, screenH/2-4-(2*8)+(i*8)+1, text[i-1].c_str(), 0);
				screen::print(0, screenW/2-(text[i-1].size()*6/2), screenH/2-4-(2*8)+(i*8), text[i-1].c_str());
			}
			break;

		case 4:
		case 6:
			screenW = 256/(textZoom/100);
			screenH = 192/(textZoom/100);

			screen::scaleView(0, textZoom/100, textZoom/100);
			screen::setAlpha(fade);
			for (int i=1; i<=2; i++)
			{
				if (fade > 0)
				{
					if (fade < 100)
					{
						screen::print(0, screenW/2-(text[i+1].size()*6/2), screenH/2-4-(2*8)+(i*8), text[i+1].c_str());
						screen::print(0, screenW/2-(text[i+1].size()*6/2)+1, screenH/2-4-(2*8)+(i*8)+1, text[i+1].c_str(), 0);
					}
					else
					{
						screen::print(0, screenW/2-(text[i+1].size()*6/2)+1, screenH/2-4-(2*8)+(i*8)+1, text[i+1].c_str(), 0);
						screen::print(0, screenW/2-(text[i+1].size()*6/2), screenH/2-4-(2*8)+(i*8), text[i+1].c_str());
					}
				}
			}
			screen::resetView();
			screen::setAlpha(100);
			break;

		case 5:
			screenW = 256;
			screenH = 192;

			for (int i=1; i<=2; i++)
			{
				screen::print(0, screenW/2-(text[i+1].size()*6/2)+1, screenH/2-4-(2*8)+(i*8)+1, text[i+1].c_str(), 0);
				screen::print(0, screenW/2-(text[i+1].size()*6/2), screenH/2-4-(2*8)+(i*8), text[i+1].c_str());
			}
			break;

		case 9:
			if (m_mainmenu->fadeAlpha <= 0)
			{
				screen::setAlpha(fade);
				screen::drawFillRect(SCREEN_UP, 0, 0, 256, 192, RGB15(31,31,31));
				screen::setAlpha(100);
			}
			break;
	}
}

void TitleUI::onPlay(void *pUserData)
{
	TitleUI* pSelf = (TitleUI*)pUserData;
	pSelf->m_mainmenu->switchUI(new CharSelectUI, true);
}

void TitleUI::onQuit(void *pUserData)
{
	TitleUI* pSelf = (TitleUI*)pUserData;
	pSelf->m_mainmenu->switchUI(new TitleUI, true);
}